@@ -22,8 +22,6 @@
 
 #include <unistd.h>
 
-#include <QLabel>
-#include <QLayout>
 //Added by qt3to4:
 #include <QVBoxLayout>
 
@@ -39,10 +37,11 @@
 
 #include "main.moc"
 
-#include "tzone.h"
 #include "dtime.h"
 #include "helper.h"
 
+#include <kauthaction.h>
+
 K_PLUGIN_FACTORY(KlockModuleFactory, registerPlugin<KclockModule>();)
 K_EXPORT_PLUGIN(KlockModuleFactory("kcmkclock"))
 
@@ -61,7 +60,7 @@ KclockModule::KclockModule(QWidget *parent, const QVariantList &)
   setAboutData( about );
   setQuickHelp( i18n("<h1>Date & Time</h1> This control module can be used to set the system date and"
     " time. As these settings do not only affect you as a user, but rather the whole system, you"
-    " can only change these settings when you start the Control Center as root. If you do not have"
+    " can only change these settings when you start the System Settings as root. If you do not have"
     " the root password, but feel the system time should be corrected, please contact your system"
     " administrator."));
 
@@ -75,49 +74,36 @@ KclockModule::KclockModule(QWidget *parent, const QVariantList &)
   layout->addWidget(dtime);
   connect(dtime, SIGNAL(timeChanged(bool)), this, SIGNAL(changed(bool)));
 
-  tzone = new Tzone(this);
-  layout->addWidget(tzone);
-  connect(tzone, SIGNAL(zoneChanged(bool)), this, SIGNAL(changed(bool)));
-
   setButtons(Help|Apply);
 
+  setNeedsAuthorization(true);
+  
   process = NULL;
 }
 
 void KclockModule::save()
 {
-  QStringList helperargs;
+  QVariantMap helperargs;
   dtime->save( helperargs );
-  tzone->save( helperargs );
-  QString helper = KStandardDirs::findExe( "kcmdatetimehelper" );
-  QString kdesu = KStandardDirs::findExe( "kdesu" );
-  bool ok = true;
-  if( helper.isEmpty() || kdesu.isEmpty())
-    ok = false;
-  else {
-    process = new KProcess(this);
-    *process << kdesu;
-    *process << "--attach" << QString::number(window()->winId()) << "--" << helper;
-    *process << helperargs;
-    process->start();
-    connect( process, SIGNAL( finished(int, QProcess::ExitStatus) ),
-		     SLOT( slotDateTimeHelperFinished(int) ) );
-    ok = process->waitForStarted();
-    if(!ok) {
-      delete process;
-      process = NULL;
+  
+  Action *action = authAction();
+  action->setArguments(helperargs);
+  
+  ActionReply reply = action->execute();
+  
+  if (reply.failed())
+  {
+    if (reply.type() == ActionReply::KAuthError) {
+          KMessageBox::error(this, i18n("Unable to authenticate/execute the action: %1, %2", reply.errorCode(), reply.errorDescription()));
+    } else {
+        dtime->processHelperErrors(reply.errorCode());
     }
+    
   }
-  if( !ok )
-    KMessageBox::error( this, i18n( "Failed to set system date/time/time zone."), i18n( "Date/Time Error" ));
 }
 void KclockModule::slotDateTimeHelperFinished(int exitCode)
 {
     dtime->processHelperErrors( exitCode );
-    tzone->processHelperErrors( exitCode);
-
-    process->deleteLater();
-    process = NULL;
 #if 0
   // Tell the clock applet about the change so that it can update its timezone
   QDBusInterface clock("org.kde.kicker", "/Applets/Clock", "org.kde.kicker.ClockApplet");
@@ -128,5 +114,4 @@ void KclockModule::slotDateTimeHelperFinished(int exitCode)
 void KclockModule::load()
 {
   dtime->load();
-  tzone->load();
 }
