@@ -22,18 +22,21 @@
 
 #include <QWidget>
 #include <QTextStream>
+#include <QTimer>
 #include "ui_outputconfigbase.h"
 #include "randr.h"
 #include "randroutput.h"
 
 class RandROutput;
-class OutputGraphicsItem;
+
+class OutputConfig;
+typedef QList<OutputConfig*> OutputConfigList;
 
 class OutputConfig : public QWidget, public Ui::OutputConfigBase 
 {
 	Q_OBJECT
 public:
-	OutputConfig(QWidget *parent, RandROutput *output, OutputGraphicsItem *item);
+	OutputConfig(QWidget *parent, RandROutput *output, OutputConfigList preceding);
 	~OutputConfig();
 	
 	/** Enumeration describing two related outputs (i.e. VGA LeftOf TMDS) */
@@ -48,15 +51,17 @@ public:
 	// NOTE: I'd love to have used Above and Below but Xlib already defines them
 	// and that confuses GCC.
 
+	bool isActive() const;
 	QPoint position(void) const;
 	QSize resolution(void) const;
+	QRect rect() const;
 	float refreshRate(void) const;
 	int rotation(void) const;
 	
 	static QString positionName(Relation position);
 	RandROutput *output(void) const;
 
-	bool hasPendingChanges(void) const;
+	bool hasPendingChanges( const QPoint& normalizePos ) const;
 	
 public slots:
 	void load();
@@ -65,6 +70,7 @@ protected slots:
 	void setConfigDirty(void);
 	
 	void updatePositionList(void);
+	void updatePositionListDelayed(void);
 	void updateRotationList(void);
 	void updateSizeList(void);
 	void updateRateList(void);
@@ -79,12 +85,16 @@ signals:
 
 
 private:
+	static bool isRelativeTo( QRect rect, QRect to, Relation rel );
 	int m_changes;
 	bool m_changed;
 	QPoint m_pos;
+	QTimer updatePositionListTimer;
 	
 	RandROutput *m_output;
-	OutputGraphicsItem *m_item;
+	// List of configs shown before this one. Relative positions may be given only
+	// relative to these in order to avoid cycles.
+	OutputConfigList precedingOutputConfigs;
 };
 
 
