@@ -242,7 +242,7 @@ struct display {
 
 	/* basic display information */
 	char *name;                 /* DISPLAY name -- also referenced in hstent */
-	char *class2;               /* display class (may be NULL) */
+	char *class2;               /* display class (may be 0) */
 	int displayType;            /* location/origin/lifetime */
 	CfgArr cfg;                 /* config data array */
 
@@ -284,10 +284,7 @@ struct display {
 
 	int idleTimeout;            /* abort login after that time */
 
-	unsigned short *authNameLens;  /* authorization protocol name lens */
-
 	/* information potentially derived from resources */
-	int authNameNum;            /* number of protocol names */
 	Xauth **authorizations;     /* authorization data */
 	int authNum;                /* number of authorizations */
 	char *authFile;             /* file to store authorization in */
@@ -303,10 +300,11 @@ struct display {
 #define dTransient      0	/* display removed when session exits */
 
 #ifdef XDMCP
-#define d_origin     8
+#define d_origin     24
 #else
-#define d_origin     0 /* clever, huh? :) */
+#define d_origin     16 /* clever, huh? :) */
 #endif
+#define dFromCommand   16	/* started via command socket */
 #define dFromXDMCP      8	/* started with XDMCP */
 #define dFromFile       0	/* started via entry in servers file */
 
@@ -375,12 +373,12 @@ extern char *progname;
 extern char *prog;
 extern time_t now;
 extern SdRec sdRec;
-void startDisplay( struct display *d );
 void startDisplayP2( struct display *d );
 void stopDisplay( struct display *d );
 #if !defined(HAVE_SETPROCTITLE) && !defined(NOXDMTITLE)
 void setproctitle( const char *fmt, ... );
 #endif
+void wakeDisplays( void );
 void switchToX( struct display *d );
 void setNLogin( struct display *d,
                 const char *nuser, const char *npass, const char *nargs,
@@ -562,6 +560,7 @@ int reader( int fd, void *buf, int len );
 int writer( int fd, const void *buf, int len );
 int fGets( char *buf, int max, FILE *f );
 time_t mTime( const char *fn );
+int hexToBinary( char *out, const char *in );
 void listSessions( int flags, struct display *d, void *ctx,
                    void (*emitXSess)( struct display *, struct display *, void * ),
                    void (*emitTTYSess)( STRUCTUTMP *, struct display *, void * ) );
