@@ -72,7 +72,7 @@ sigAlarm( int )
 
 GreeterApp::GreeterApp( int &argc, char **argv ) :
 	inherited( argc, argv ),
-	regrabPtr( false ), regrabKbd( false ),
+	regrabPtr( false ), regrabKbd( false ), initalBusy( true ),
 	dragWidget( 0 )
 {
 	pingInterval = _isLocal ? 0 : _pingInterval;
@@ -88,6 +88,23 @@ GreeterApp::GreeterApp( int &argc, char **argv ) :
 		pingTimerId = 0;
 }
 
+void
+GreeterApp::markBusy()
+{
+	if (initalBusy)
+		initalBusy = false;
+	else
+		setCursor( QX11Info::display(), desktop()->winId(), XC_watch );
+	setOverrideCursor( Qt::WaitCursor );
+}
+
+void
+GreeterApp::markReady()
+{
+	restoreOverrideCursor();
+	setCursor( QX11Info::display(), desktop()->winId(), XC_left_ptr );
+}
+
 void
 GreeterApp::timerEvent( QTimerEvent *ev )
 {
@@ -104,27 +121,31 @@ GreeterApp::x11EventFilter( XEvent * ev )
 {
 	KeySym sym;
 
+	if (_grabInput) {
+		switch (ev->type) {
+		case FocusIn:
+		case FocusOut:
+			if (ev->xfocus.mode == NotifyUngrab) {
+				if (!regrabKbd) {
+					secureKeyboard( QX11Info::display() );
+					regrabKbd = true;
+				}
+			} else
+				regrabKbd = false;
+			return false;
+		case EnterNotify:
+		case LeaveNotify:
+			if (ev->xcrossing.mode == NotifyUngrab) {
+				if (!regrabPtr) {
+					securePointer( QX11Info::display() );
+					regrabPtr = true;
+				}
+			} else
+				regrabPtr = false;
+			return false;
+		}
+	}
 	switch (ev->type) {
-	case FocusIn:
-	case FocusOut:
-		if (ev->xfocus.mode == NotifyUngrab) {
-			if (!regrabKbd) {
-				secureKeyboard( QX11Info::display() );
-				regrabKbd = true;
-			}
-		} else
-			regrabKbd = false;
-		break;
-	case EnterNotify:
-	case LeaveNotify:
-		if (ev->xcrossing.mode == NotifyUngrab) {
-			if (!regrabPtr) {
-				securePointer( QX11Info::display() );
-				regrabPtr = true;
-			}
-		} else
-			regrabPtr = false;
-		break;
 	case KeyPress:
 		sym = XLookupKeysym( &ev->xkey, 0 );
 		if (sym != XK_Return && !IsModifierKey( sym ))
@@ -225,6 +246,12 @@ goodLocale( const char *var )
 	return False;
 }
 
+static uint
+qHash(const QSize &sz)
+{
+	return (sz.width() << 12) ^ sz.height();
+}
+
 int
 main( int argc ATTR_UNUSED, char **argv )
 {
@@ -363,20 +390,30 @@ main( int argc ATTR_UNUSED, char **argv )
 	gSendInt( G_Ready );
 
 	if (themer) {
-		QPixmap pm( dw->size() );
+		// Group by size to avoid rescaling images repeatedly
+		QHash<QSize, QList<int> > scrns;
 		for (int i = 0; i < dw->numScreens(); i++)
-			themer->paintBackground( &pm, dw->screenGeometry( i ), i == _greeterScreen );
+			scrns[dw->screenGeometry( i ).size()] << i;
+		QPixmap pm( dw->size() );
+		QSize gsz = dw->screenGeometry( _greeterScreen ).size();
+		// Paint these first, as throwing away their images does not hurt
+		QHash<QSize, QList<int> >::ConstIterator it;
+		for (it = scrns.constBegin(); it != scrns.constEnd(); ++it)
+			if (it.key() != gsz)
+				foreach (int i, it.value())
+					themer->paintBackground( &pm, dw->screenGeometry( i ), false );
+		// If we are lucky, these will use the same images as the greeter
+		foreach (int i, scrns.value(gsz))
+			if (i != _greeterScreen)
+				themer->paintBackground( &pm, dw->screenGeometry( i ), false );
+		// Paint the greeter background last - it will be re-used.
+		themer->paintBackground( &pm, dw->screenGeometry( _greeterScreen ), true );
 		QPalette palette;
 		palette.setBrush( dw->backgroundRole(), QBrush( pm ) );
 		dw->setPalette( palette );
-		dw->setAutoFillBackground( true );
-		dw->setAttribute( Qt::WA_PaintOnScreen );
-		dw->show();
-		dw->repaint();
+		XClearWindow( dpy, dw->winId() );
 	}
 
-	setCursor( dpy, app.desktop()->winId(), XC_left_ptr );
-
 	int rslt = ex_exit;
 	for (;;) {
 		int cmd = gRecvInt();
@@ -387,12 +424,14 @@ main( int argc ATTR_UNUSED, char **argv )
 			int how = gRecvInt(), uid = gRecvInt();
 			QString os = qString( gRecvStr() );
 			gSet( 0 );
+			app.markReady();
 			KDMSlimShutdown::externShutdown( how, os, uid, true );
 			gSendInt( G_Ready );
 			break;
 		}
 
 		if (cmd == G_ErrorGreet) {
+			app.markReady();
 			if (KGVerify::handleFailVerify( qApp->desktop()->screen( _greeterScreen ), true ))
 				break;
 			_autoLoginDelay = 0;
@@ -400,7 +439,7 @@ main( int argc ATTR_UNUSED, char **argv )
 		}
 
 		KProcess *proc2 = 0;
-		app.setOverrideCursor( Qt::WaitCursor );
+		app.markBusy();
 		FDialog *dialog;
 #ifdef XDMCP
 		if (cmd == G_Choose) {
@@ -423,7 +462,7 @@ main( int argc ATTR_UNUSED, char **argv )
 				proc2->start();
 			}
 		}
-		app.restoreOverrideCursor();
+		QObject::connect( dialog, SIGNAL(ready()), &app, SLOT(markReady()) );
 		debug( "entering event loop\n" );
 		rslt = dialog->exec();
 		debug( "left event loop\n" );
