@@ -45,6 +45,20 @@ class QMimeData;
 class HistoryItem;
 class KlipperSessionManager;
 
+class KlipperEmptyDetector : public QWidget
+{
+Q_OBJECT
+public:
+    KlipperEmptyDetector();
+signals:
+    void changed( QClipboard::Mode m );
+protected:
+    virtual bool x11Event( XEvent* e );
+private:
+    int m_xfixes_event_base;
+    unsigned int m_xa_clipboard;
+};
+
 class Klipper : public QObject
 {
   Q_OBJECT
@@ -220,6 +234,7 @@ private:
     QString cycleText() const;
     KlipperSessionManager* m_session_managed;
     KActionCollection *m_collection;
+    KlipperEmptyDetector m_empty_detector;
 };
 
 #endif
