@@ -24,6 +24,7 @@
 
 #include <math.h>
 
+#include <QAction>
 #include <QTimeLine>
 #include <QDebug>
 #include <QtGlobal>
@@ -52,7 +53,7 @@ void shadowBlur(QImage &image, int radius, const QColor &color);
 
 int ResultItem::s_fontHeight = 0;
 
-ResultItem::ResultItem(const Plasma::QueryMatch &match, QGraphicsWidget *parent)
+ResultItem::ResultItem(const Plasma::QueryMatch &match, Plasma::RunnerManager *runnerManager, QGraphicsWidget *parent)
     : QGraphicsWidget(parent),
       m_match(0),
       m_configButton(0),
@@ -60,7 +61,10 @@ ResultItem::ResultItem(const Plasma::QueryMatch &match, QGraphicsWidget *parent)
       m_index(-1),
       m_highlightTimerId(0),
       m_mouseHovered(false),
-      m_configWidget(0)
+      m_configWidget(0),
+      m_actionsWidget(0),
+      m_actionsLayout(0),
+      m_runnerManager(runnerManager)
 {
     setFlag(QGraphicsItem::ItemIsFocusable);
     setFlag(QGraphicsItem::ItemIsSelectable);
@@ -84,34 +88,137 @@ ResultItem::~ResultItem()
 {
 }
 
+QGraphicsWidget* ResultItem::arrangeTabOrder(QGraphicsWidget* last)
+{
+    QGraphicsWidget *sceneWidget = static_cast<QGraphicsWidget*>(parent());
+    sceneWidget->setTabOrder(last, this);
+    QGraphicsWidget *currentWidget = this;
+
+    if (m_configButton) {
+        sceneWidget->setTabOrder(this, m_configButton);
+        currentWidget = m_configButton;
+        if (m_configWidget) {
+            sceneWidget->setTabOrder(m_configButton, m_configWidget);
+            currentWidget = m_configWidget;
+        }
+    }
+    if (m_actionsWidget) {
+        for (int i = 0; i < m_actionsLayout->count(); ++i) {
+            QGraphicsWidget *button = static_cast<QGraphicsWidget*>(m_actionsLayout->itemAt(i));
+            sceneWidget->setTabOrder(currentWidget, button);
+            currentWidget = button;
+        }
+    }
+    return currentWidget;
+}
+
 void ResultItem::setMatch(const Plasma::QueryMatch &match)
 {
     m_match = match;
     m_icon = KIcon(match.icon());
 
     if (m_configWidget) {
+        if (scene()) {
+            scene()->removeItem(m_configWidget);
+        }
+
         delete m_configWidget;
         m_configWidget = 0;
     }
 
+    if (m_actionsWidget) {
+        if (scene()) {
+            scene()->removeItem(m_actionsWidget);
+        }
+
+        delete m_actionsWidget;
+        m_actionsWidget = 0;
+    }
+
     //kDebug() << match.hasConfigurationInterface();
     if (match.hasConfigurationInterface()) {
-        m_configButton = new Plasma::ToolButton(this);
-        m_configButton->setIcon(KIcon("configure"));
-        m_configButton->show();
-        m_configButton->resize(m_configButton->effectiveSizeHint(Qt::MinimumSize,
-                                                        QSize(KIconLoader::SizeSmall,
-                                                              KIconLoader::SizeSmall)));
-        connect(m_configButton, SIGNAL(clicked()), this, SLOT(showConfig()));
-    } else {
+        if (!m_configButton) {
+            m_configButton = new Plasma::ToolButton(this);
+            m_configButton->setIcon(KIcon("configure"));
+            m_configButton->show();
+            m_configButton->resize(m_configButton->effectiveSizeHint(Qt::MinimumSize,
+                                                                     QSize(KIconLoader::SizeSmall,
+                                                                           KIconLoader::SizeSmall)));
+            connect(m_configButton, SIGNAL(clicked()), this, SLOT(showConfig()));
+            m_configButton->installEventFilter(this);
+        }
+    } else if (m_configButton) {
+        if (scene()) {
+            scene()->removeItem(m_configButton);
+        }
+
         delete m_configButton;
         m_configButton = 0;
     }
 
+    setupActions();
     calculateSize();
     update();
 }
 
+void ResultItem::setupActions()
+{
+    //kDebug();
+    QList<QAction*> actionList = m_runnerManager->actionsForMatch(m_match);
+
+    if (!actionList.isEmpty()) {
+        m_actionsWidget = new QGraphicsWidget(this);
+        m_actionsLayout = new QGraphicsLinearLayout(Qt::Horizontal, m_actionsWidget);
+
+        foreach (QAction* action, actionList) {
+            Plasma::ToolButton * actionButton = new Plasma::ToolButton(m_actionsWidget);
+            actionButton->setFlag(QGraphicsItem::ItemIsFocusable);
+            actionButton->setAction(action);
+            actionButton->show();
+            actionButton->resize(actionButton->effectiveSizeHint(Qt::MinimumSize,
+                                                        QSize(KIconLoader::SizeSmall,
+                                                              KIconLoader::SizeSmall)));
+            m_actionsLayout->addItem(actionButton);
+            connect(actionButton, SIGNAL(clicked()), this , SIGNAL(actionTriggered()));
+            actionButton->installEventFilter(this);
+        }
+        m_actionsWidget->show();
+    }
+}
+
+bool ResultItem::eventFilter(QObject *obj, QEvent *event)
+{
+    Plasma::ToolButton* actionButton = static_cast<Plasma::ToolButton*>(obj);
+
+    if (actionButton) {
+        if (event->type() == QEvent::GraphicsSceneHoverEnter) {
+            if (scene()) {
+                scene()->setFocusItem(actionButton);
+            }
+        } else if (event->type() == QEvent::FocusIn) {
+            focusInEvent(static_cast<QFocusEvent*>(event));
+            actionButton->setAutoRaise(false);
+            update();
+        } else if (event->type() == QEvent::GraphicsSceneHoverLeave || event->type() == QEvent::FocusOut) {
+            actionButton->setAutoRaise(true);
+            update();
+        } else if (event->type() == QEvent::KeyPress) {
+            QKeyEvent* keyEvent = static_cast<QKeyEvent*>(event);
+            if (keyEvent->key() == Qt::Key_Enter || keyEvent->key() == Qt::Key_Return) {
+                if (actionButton->action()) {
+                        actionButton->action()->trigger();
+                        emit actionTriggered();
+                    } else {
+                        showConfig();
+                    }
+                return true;
+            }
+        }
+    }
+
+    return false;
+}
+
 QString ResultItem::id() const
 {
     return m_match.id();
@@ -134,6 +241,17 @@ QString ResultItem::name() const
 
 QString ResultItem::description() const
 {
+    if (!scene()) {
+        return QString();
+    }
+
+    Plasma::ToolButton* actionButton = qobject_cast<Plasma::ToolButton*>(static_cast<QGraphicsWidget*>(scene()->focusItem()));
+
+    //if a button is focused and it  belongs to the item
+    if (actionButton && actionButton->parentWidget() == m_actionsWidget) {
+        return actionButton->text();
+    }
+
     return m_match.subtext();
 }
 
@@ -326,7 +444,11 @@ void ResultItem::hoverEnterEvent(QGraphicsSceneHoverEvent *e)
 {
     QGraphicsItem::hoverEnterEvent(e);
     setFocus(Qt::MouseFocusReason);
-    scene()->clearSelection();
+
+    if (scene()) {
+        scene()->clearSelection();
+    }
+
     setSelected(true);
 }
 
@@ -355,12 +477,17 @@ bool ResultItem::mouseHovered() const
 void ResultItem::focusInEvent(QFocusEvent * event)
 {
     QGraphicsWidget::focusInEvent(event);
+    //kDebug() << hasFocus();
     setZValue(1);
 
     m_mouseHovered = (event->reason() == Qt::MouseFocusReason);
 
-    scene()->clearSelection();
+    if (scene()) {
+        scene()->clearSelection();
+    }
+
     setSelected(true);
+    emit ensureVisibility(this);
 
     if (!m_highlightTimerId) {
         m_highlightTimerId = startTimer(TIMER_INTERVAL);
@@ -392,7 +519,7 @@ void ResultItem::keyPressEvent(QKeyEvent *event)
 
 QVariant ResultItem::itemChange(GraphicsItemChange change, const QVariant &value)
 {
-    if (change == QGraphicsItem::ItemSceneHasChanged && scene()) {
+    if (change == QGraphicsItem::ItemSceneHasChanged) {
         calculateSize();
     }
 
@@ -416,6 +543,10 @@ void ResultItem::changeEvent(QEvent *event)
 void ResultItem::showConfig()
 {
     if (m_configWidget) {
+        if (scene()) {
+            scene()->removeItem(m_configWidget);
+        }
+
         delete m_configWidget;
         m_configWidget = 0;
     } else {
@@ -425,6 +556,8 @@ void ResultItem::showConfig()
         m_configWidget = new QGraphicsProxyWidget(this);
         m_configWidget->setWidget(w);
         m_configWidget->show();
+        QGraphicsWidget* sceneWidget = parentWidget();
+        sceneWidget->setTabOrder(m_configButton, m_configWidget);
     }
 
     calculateSize();
@@ -434,7 +567,7 @@ void ResultItem::showConfig()
 void ResultItem::calculateSize()
 {
     if (scene()) {
-        calculateSize(scene()->width(),scene()->height());
+        calculateSize(scene()->width(), scene()->height());
     }
 }
 
@@ -490,6 +623,19 @@ void ResultItem::calculateSize(int sceneWidth, int sceneHeight)
                                newSize.height() - m_configWidget->size().height() - bottom);
     }
 
+    if (m_actionsWidget) {
+        m_actionsWidget->setMaximumWidth(newSize.width()/2);
+        m_actionsWidget->adjustSize();
+        QSizeF s = m_actionsWidget->size();
+
+        if (QApplication::layoutDirection() == Qt::RightToLeft) {
+            m_actionsWidget->setPos(left, newSize.height() - s.height() - bottom);
+        } else {
+            m_actionsWidget->setPos(newSize.width() - s.width() - right,
+                                   newSize.height() - s.height() - bottom);
+        }
+    }
+
     resize(newSize);
 }
 
