@@ -61,7 +61,7 @@ class ResultItem : public QGraphicsWidget
     Q_OBJECT
 
 public:
-    ResultItem(const Plasma::QueryMatch &match, QGraphicsWidget *parent);
+    ResultItem(const Plasma::QueryMatch &match, Plasma::RunnerManager *runnerManager, QGraphicsWidget *parent);
 
     void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget = 0);
     void setMatch(const Plasma::QueryMatch &match);
@@ -83,6 +83,7 @@ public:
     bool mouseHovered() const;
     void calculateSize();
     void calculateSize(int sceneWidth, int sceneHeight);
+    QGraphicsWidget* arrangeTabOrder(QGraphicsWidget* last);
 
     static bool compare(const ResultItem *one, const ResultItem *other);
     bool operator<(const ResultItem &other) const;
@@ -96,6 +97,8 @@ signals:
     void hoverEnter(ResultItem *item);
     void hoverLeave(ResultItem *item);
     void sizeChanged(ResultItem *item);
+    void actionTriggered();
+    void ensureVisibility(QGraphicsItem *item);
 
 protected:
     void hoverLeaveEvent(QGraphicsSceneHoverEvent *e);
@@ -109,6 +112,8 @@ protected:
     void changeEvent(QEvent *event);
     void resizeEvent(QGraphicsSceneResizeEvent *);
     void drawIcon(QPainter *painter, const QRect &iRect, const QPixmap &p);
+    void setupActions();
+    bool eventFilter(QObject *obj, QEvent *event);
 
 protected slots:
     void showConfig();
@@ -129,6 +134,9 @@ private:
     int m_highlightTimerId;
     bool m_mouseHovered;
     QGraphicsProxyWidget *m_configWidget;
+    QGraphicsWidget *m_actionsWidget;
+    QGraphicsLinearLayout *m_actionsLayout;
+    Plasma::RunnerManager *m_runnerManager;
 
     static int s_fontHeight;
 };
