@@ -51,6 +51,7 @@ class ResultScene : public QGraphicsScene
         void run(ResultItem* item) const;
         QSize minimumSizeHint() const;
         void setItemsAcceptHoverEvents(bool enable);
+        bool itemsAcceptHoverEvents();
 //        Plasma::RunnerManager* manager() const;
 
 
@@ -66,6 +67,7 @@ class ResultScene : public QGraphicsScene
         void itemHoverLeave(ResultItem *item);
         void matchCountChanged(int count);
         void ensureVisibility(QGraphicsItem *item);
+        void actionTriggered();
 
     protected:
         void keyPressEvent(QKeyEvent * keyEvent);
@@ -76,6 +78,8 @@ class ResultScene : public QGraphicsScene
         void selectPreviousItem();
         void selectNextItem();
 
+        ResultItem* currentlyFocusedItem() const;
+
         ResultItem* addQueryMatch(const Plasma::QueryMatch &match, bool useAnyId);
 
         bool canMoveItemFocus() const;
@@ -104,6 +108,7 @@ class ResultScene : public QGraphicsScene
         qreal m_itemMarginBottom;
 
         QWidget *m_focusBase;
+        bool m_itemsAcceptHoverEvents;
 };
 
 #endif
