@@ -85,7 +85,8 @@ PasswordDlg::PasswordDlg(LockProcess *parent, GreeterPluginHandle *plugin, const
     : KDialog(parent, Qt::X11BypassWindowManagerHint),
       mPlugin( plugin ),
       mCapsLocked(-1),
-      mUnlockingFailed(false)
+      mUnlockingFailed(false),
+      sNot(0)
 {
     QWidget* w = mainWidget();
 
@@ -317,9 +318,14 @@ void PasswordDlg::reapVerify()
 {
     sNot->setEnabled( false );
     sNot->deleteLater();
+    sNot = 0;
     ::close( sFd );
     int status;
-    ::waitpid( sPid, &status, 0 );
+    while (::waitpid( sPid, &status, 0 ) < 0)
+        if (errno != EINTR) { // This should not happen ...
+            cantCheck();
+            return;
+        }
     if (WIFEXITED(status))
         switch (WEXITSTATUS(status)) {
         case AuthOk:
@@ -434,6 +440,8 @@ void PasswordDlg::gplugStart()
     int sfd[2];
     char fdbuf[16];
 
+    if (sNot)
+        return;
     if (::socketpair(AF_LOCAL, SOCK_STREAM, 0, sfd)) {
         cantCheck();
         return;
