@@ -174,6 +174,7 @@ namespace Oxygen
     glowAnimation().data()->setDuration( configuration_.animationsDuration() );
     titleAnimationData_.data()->setDuration( configuration_.animationsDuration() );
     itemData_.animation().data()->setDuration( configuration_.animationsDuration() );
+    itemData_.setAnimationsEnabled( useAnimations() );
 
     // reset title transitions
     titleAnimationData_.data()->reset();
@@ -419,14 +420,6 @@ namespace Oxygen
   QRect OxygenClient::titleBoundingRect( const QFont& font, QRect rect, const QString& caption ) const
   {
 
-    // check bounding rect against titleRect
-    if( titleRectCanConflict() ) 
-    {
-      QRect titleRect( OxygenClient::titleRect() );
-      if( titleRect.left() > rect.left() ) { rect.setLeft( titleRect.left() ); }
-      if( titleRect.right() < rect.right() ) { rect.setRight( titleRect.right() ); }
-    }
-
     // get title bounding rect
     QRect boundingRect = QFontMetrics( font ).boundingRect( rect, configuration().titleAlignment() | Qt::AlignVCenter, caption );
 
@@ -819,15 +812,35 @@ namespace Oxygen
     if( titleAnimationData_.data()->isDirty() )
     {
 
-        // contrast pixmap
-        titleAnimationData_.data()->setPixmaps(
-            rect,
-            renderTitleText( rect, caption(), color ),
-            renderTitleText( rect, caption(), contrast ) );
-
+        // clear dirty flags
         titleAnimationData_.data()->setDirty( false );
-        titleAnimationData_.data()->startAnimation();
-        renderTitleText( painter, rect, color, contrast );
+
+        // finish current animation if running
+        if( titleAnimationData_.data()->isAnimated() )
+        { titleAnimationData_.data()->finishAnimation(); }
+
+        if( !titleAnimationData_.data()->isLocked() )
+        {
+
+            // set pixmaps
+            titleAnimationData_.data()->setPixmaps(
+                rect,
+                renderTitleText( rect, caption(), color ),
+                renderTitleText( rect, caption(), contrast ) );
+
+            titleAnimationData_.data()->startAnimation();
+            renderTitleText( painter, rect, color, contrast );
+
+        } else if( !caption().isEmpty() ) {
+
+            renderTitleText( painter, rect, caption(), color, contrast );
+
+        }
+
+        // lock animations (this must be done whether or not
+        // animation was actually started, in order to extend locking
+        // every time title get changed too rapidly
+        titleAnimationData_.data()->lockAnimations();
 
     } else if( titleAnimationData_.data()->isAnimated() ) {
 
@@ -927,20 +940,25 @@ namespace Oxygen
     const QList< ClientGroupItem >& items( clientGroupItems() );
     QString caption( itemCount == 1 ? OxygenClient::caption() : items[index].title() );
 
-    // make sure title rect never conflicts with decoration buttons
-    if( titleRectCanConflict() ) 
-    { textRect = titleBoundingRect( painter->font(), textRect, caption ); }
-    
+    // always make sure that titleRect never conflicts with window buttons
+    QRect titleRect( OxygenClient::titleRect() );
+    if( titleRect.left() > textRect.left() ) { textRect.setLeft( titleRect.left() ); }
+    if( titleRect.right() < textRect.right() ) { textRect.setRight( titleRect.right() ); }
+
     // title outline
     if( itemCount == 1 )
     {
-        
+
       if( itemData_.isAnimated() ) {
 
+        textRect = titleBoundingRect( painter->font(), textRect, caption );
         renderTitleOutline( painter, item.boundingRect_, palette );
 
       } else if( (isActive()||glowIsAnimated()) && configuration().drawTitleOutline() ) {
 
+        // adjust textRect
+        textRect = titleBoundingRect( painter->font(), textRect, caption );
+
         // adjusts boundingRect accordingly
         QRect boundingRect( item.boundingRect_ );
         boundingRect.setLeft( textRect.left() - layoutMetric( LM_TitleBorderLeft ) );
