@@ -32,7 +32,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <QFile>
 
 #include "kdebug.h"
-#include "klibloader.h"
+#include "klibrary.h"
 #include "kdesktopfile.h"
 #include "kconfiggroup.h"
 #include "kstandarddirs.h"
@@ -208,6 +208,12 @@ void EffectsHandlerImpl::windowUserMovedResized( EffectWindow* c, bool first, bo
         ep.second->windowUserMovedResized( c, first, last );
     }
 
+void EffectsHandlerImpl::windowMoveResizeGeometryUpdate( EffectWindow* c, const QRect& geometry )
+    {
+    foreach( const EffectPair &ep, loaded_effects )
+        ep.second->windowMoveResizeGeometryUpdate( c, geometry );
+    }
+
 void EffectsHandlerImpl::windowOpacityChanged( EffectWindow* c, double old_opacity )
     {
         if (!c)
@@ -255,6 +261,24 @@ void EffectsHandlerImpl::windowUnminimized( EffectWindow* c )
         ep.second->windowUnminimized( c );
     }
 
+void EffectsHandlerImpl::clientGroupItemSwitched( EffectWindow* from, EffectWindow* to )
+    {
+    foreach( const EffectPair &ep, loaded_effects )
+        ep.second->clientGroupItemSwitched( from, to );
+    }
+
+void EffectsHandlerImpl::clientGroupItemAdded( EffectWindow* from, EffectWindow* to )
+    {
+    foreach( const EffectPair &ep, loaded_effects )
+        ep.second->clientGroupItemAdded( from, to );
+    }
+
+void EffectsHandlerImpl::clientGroupItemRemoved( EffectWindow* c, EffectWindow* group )
+    {
+    foreach( const EffectPair &ep, loaded_effects )
+        ep.second->clientGroupItemRemoved( c, group );
+    }
+
 void EffectsHandlerImpl::desktopChanged( int old )
     {
     foreach( const EffectPair &ep, loaded_effects )
@@ -295,6 +319,12 @@ void EffectsHandlerImpl::tabBoxUpdated()
         ep.second->tabBoxUpdated();
     }
 
+void EffectsHandlerImpl::tabBoxKeyEvent( QKeyEvent* event )
+    {
+    foreach( const EffectPair &ep, loaded_effects )
+        ep.second->tabBoxKeyEvent( event );
+    }
+
 void EffectsHandlerImpl::setActiveFullScreenEffect( Effect* e )
     {
     fullscreen_effect = e;
@@ -347,7 +377,7 @@ void EffectsHandlerImpl::grabbedKeyboardEvent( QKeyEvent* e )
         keyboard_grab_effect->grabbedKeyboardEvent( e );
     }
 
-const void* EffectsHandlerImpl::getProxy( QString name )
+void* EffectsHandlerImpl::getProxy( QString name )
     {
     // All effects start with "kwin4_effect_", prepend it to the name
     name.prepend( "kwin4_effect_" );
@@ -409,10 +439,15 @@ EffectWindow* EffectsHandlerImpl::activeWindow() const
     return Workspace::self()->activeClient() ? Workspace::self()->activeClient()->effectWindow() : NULL;
     }
 
-void EffectsHandlerImpl::moveWindow( EffectWindow* w, const QPoint& pos )
+void EffectsHandlerImpl::moveWindow( EffectWindow* w, const QPoint& pos, bool snap, double snapAdjust )
     {
     Client* cl = dynamic_cast< Client* >( static_cast<EffectWindowImpl*>(w)->window());
-    if( cl && cl->isMovable())
+    if (!cl || !cl->isMovable())
+        return;
+
+    if (snap)
+        cl->move(Workspace::self()->adjustClientPosition(cl, pos, true, snapAdjust));
+    else
         cl->move( pos );
     }
 
@@ -423,6 +458,19 @@ void EffectsHandlerImpl::windowToDesktop( EffectWindow* w, int desktop )
         Workspace::self()->sendClientToDesktop( cl, desktop, true );
     }
 
+void EffectsHandlerImpl::windowToScreen( EffectWindow* w, int screen )
+    {
+    Client* cl = dynamic_cast< Client* >( static_cast<EffectWindowImpl*>(w)->window());
+    if( cl && !cl->isDesktop() && !cl->isDock() && !cl->isTopMenu())
+        Workspace::self()->sendClientToScreen( cl, screen );
+    }
+
+void EffectsHandlerImpl::setShowingDesktop( bool showing )
+    {
+    Workspace::self()->setShowingDesktop( showing );
+    }
+
+
 int EffectsHandlerImpl::currentDesktop() const
     {
     return Workspace::self()->currentDesktop();
@@ -829,7 +877,7 @@ unsigned long EffectsHandlerImpl::xrenderBufferPicture()
 KLibrary* EffectsHandlerImpl::findEffectLibrary( KService* service )
     {
     QString libname = service->library();
-    KLibrary* library = KLibLoader::self()->library(libname);
+    KLibrary* library = new KLibrary(libname);
     if( !library )
         {
         kError( 1212 ) << "couldn't open library for effect '" <<
@@ -1268,6 +1316,12 @@ QByteArray EffectWindowImpl::readProperty( long atom, long type, int format ) co
         }
     }
 
+void EffectWindowImpl::deleteProperty(long int atom) const
+{
+    XDeleteProperty( QX11Info::display(), window()->window(), atom );
+}
+
+
 bool EffectWindowImpl::isMovable() const
     {
     if( Client* c = dynamic_cast< Client* >( toplevel ))
@@ -1442,6 +1496,56 @@ WindowQuadList EffectWindowImpl::buildQuads( bool force ) const
     return sceneWindow()->buildQuads( force );
     }
 
+void EffectWindowImpl::minimize() const
+    {
+    if( Client* c = dynamic_cast< Client* >( toplevel ))
+        {
+        c->minimize();
+        }
+    }
+
+void EffectWindowImpl::unminimize() const
+    {
+    if( Client* c = dynamic_cast< Client* >( toplevel ))
+        {
+        c->unminimize();
+        }
+    }
+
+void EffectWindowImpl::closeWindow() const
+    {
+    if( Client* c = dynamic_cast< Client* >( toplevel ))
+        {
+        c->closeWindow();
+        }
+    }
+
+bool EffectWindowImpl::visibleInClientGroup() const
+    {
+    if( Client* c = dynamic_cast< Client* >( toplevel ))
+        {
+        if( !c->clientGroup() )
+            return true;
+        return c == c->clientGroup()->visible();
+        }
+    return false;
+    }
+
+void EffectWindowImpl::setData( int role, const QVariant &data )
+    {
+    if ( !data.isNull() )
+        dataMap[ role ] = data;
+    else
+        dataMap.remove( role );
+    }
+
+QVariant EffectWindowImpl::data( int role ) const
+    {
+    if( !dataMap.contains( role ) )
+        return QVariant();
+    return dataMap[ role ];
+    }
+
 EffectWindow* effectWindow( Toplevel* w )
     {
     EffectWindowImpl* ret = w->effectWindow();
