@@ -18,6 +18,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *********************************************************************/
 #include "cube.h"
+#include "cube_inside.h"
 
 #include <kaction.h>
 #include <kactioncollection.h>
@@ -50,6 +51,7 @@ CubeEffect::CubeEffect()
     , cube_painting( false )
     , keyboard_grab( false )
     , schedule_close( false )
+    , painting_desktop( 1 )
     , frontDesktop( 0 )
     , cubeOpacity( 1.0 )
     , opacityDesktopOnly( true )
@@ -89,6 +91,7 @@ CubeEffect::CubeEffect()
     , capListCreated( false )
     , recompileList( true )
     , glList( 0 )
+    , m_proxy( this )
     {
     desktopNameFont.setBold( true );
     desktopNameFont.setPointSize( 14 );
@@ -440,6 +443,18 @@ void CubeEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
                 glPopMatrix();
                 }
             glPopMatrix();
+
+            // call the inside cube effects
+            foreach( CubeInsideEffect* inside, m_cubeInsideEffects )
+                {
+                glPushMatrix();
+                glCallList( glList );
+                glTranslatef( rect.width()/2, rect.height()/2, -point-zTranslate );
+                glRotatef( (1-frontDesktop)*360.0f / effects->numberOfDesktops(), 0.0, 1.0, 0.0 );
+                inside->paint();
+                glPopMatrix();
+                }
+
             glCullFace( GL_BACK );
             if( mode == Cylinder )
                 {
@@ -580,6 +595,19 @@ void CubeEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
             glPopMatrix();
             }
         glPopMatrix();
+
+
+        // call the inside cube effects
+        foreach( CubeInsideEffect* inside, m_cubeInsideEffects )
+            {
+            glPushMatrix();
+            glCallList( glList );
+            glTranslatef( rect.width()/2, rect.height()/2, -point-zTranslate );
+            glRotatef( (1-frontDesktop)*360.0f / effects->numberOfDesktops(), 0.0, 1.0, 0.0 );
+            inside->paint();
+            glPopMatrix();
+            }
+
         glCullFace( GL_FRONT );
         if( mode == Cylinder )
             {
@@ -1151,6 +1179,7 @@ void CubeEffect::postPaintScreen()
 
                 // delete the GL lists
                 glDeleteLists( glList, 3 );
+                desktopNameFrame.free();
                 }
             effects->addRepaintFull();
             }
@@ -1986,6 +2015,10 @@ void CubeEffect::rotateToDesktop( int desktop )
 
 void CubeEffect::setActive( bool active )
     {
+    foreach( CubeInsideEffect* inside, m_cubeInsideEffects )
+        {
+        inside->setActive( true );
+        }
     if( active )
         {
         if( !mousePolling )
@@ -2039,7 +2072,6 @@ void CubeEffect::setActive( bool active )
             mousePolling = false;
             }
         schedule_close = true;
-        desktopNameFrame.free();
         // we have to add a repaint, to start the deactivating
         effects->addRepaintFull();
         }
@@ -2117,6 +2149,7 @@ void CubeEffect::mouseChanged( const QPoint& pos, const QPoint& oldpos, Qt::Mous
 void CubeEffect::windowInputMouseEvent( Window w, QEvent* e )
     {
     assert( w == input );
+    Q_UNUSED( w );
     QMouseEvent *mouse = dynamic_cast< QMouseEvent* >( e );
     if( mouse && mouse->type() == QEvent::MouseButtonRelease )
         {
@@ -2173,7 +2206,7 @@ void CubeEffect::tabBoxAdded( int mode )
         return;
     if( effects->activeFullScreenEffect() && effects->activeFullScreenEffect() != this )
         return;
-    if( useForTabBox && mode != TabBoxWindowsMode )
+    if( useForTabBox && mode == TabBoxDesktopListMode )
         {
         effects->refTabBox();
         tabBoxMode = true;
@@ -2185,7 +2218,10 @@ void CubeEffect::tabBoxAdded( int mode )
 void CubeEffect::tabBoxUpdated()
     {
     if( activated )
+        {
         rotateToDesktop( effects->currentTabBoxDesktop() );
+        effects->addRepaintFull();
+        }
     }
 
 void CubeEffect::tabBoxClosed()
@@ -2221,4 +2257,19 @@ void CubeEffect::sphereShortcutChanged( const QKeySequence& seq )
     sphereShortcut = KShortcut( seq );
     }
 
+void* CubeEffect::proxy()
+    {
+    return &m_proxy;
+    }
+
+void CubeEffect::registerCubeInsideEffect(CubeInsideEffect* effect)
+    {
+    m_cubeInsideEffects.append( effect );
+    }
+
+void CubeEffect::unregisterCubeInsideEffect(CubeInsideEffect* effect)
+    {
+    m_cubeInsideEffects.removeAll( effect );
+    }
+
 } // namespace
