@@ -4,6 +4,7 @@
 
 Copyright (C) 2007 Lubos Lunak <l.lunak@kde.org>
 Copyright (C) 2008 Lucas Murray <lmurray@undefinedfire.com>
+Copyright (C) 2009 Martin Gräßlin <kde@martin-graesslin.com>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -33,6 +34,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <netwm_def.h>
 #include <QEvent>
 #include <QMouseEvent>
+#include <kglobalsettings.h>
 
 namespace KWin
 {
@@ -46,6 +48,7 @@ DesktopGridEffect::DesktopGridEffect()
     , timeline()
     , keyboardGrab( false )
     , wasWindowMove( false )
+    , wasDesktopMove( false )
     , windowMove( NULL )
     , windowMoveDiff()
     , gridSize()
@@ -55,6 +58,7 @@ DesktopGridEffect::DesktopGridEffect()
     , unscaledBorder()
     , scaledSize()
     , scaledOffset()
+    , m_proxy( 0 )
     {
     // Load shortcuts
     KActionCollection* actionCollection = new KActionCollection( this );
@@ -103,6 +107,7 @@ void DesktopGridEffect::reconfigure( ReconfigureFlags )
     desktopNameAlignment = Qt::Alignment( conf.readEntry( "DesktopNameAlignment", 0 ));
     layoutMode = conf.readEntry( "LayoutMode", int( LayoutPager ));
     customLayoutRows = conf.readEntry( "CustomLayoutRows", 2 );
+    m_usePresentWindows = conf.readEntry( "PresentWindows", true );
     }
 
 //-----------------------------------------------------------------------------
@@ -110,7 +115,7 @@ void DesktopGridEffect::reconfigure( ReconfigureFlags )
 
 void DesktopGridEffect::prePaintScreen( ScreenPrePaintData& data, int time )
     {
-    if( timeline.value() != 0 || activated )
+    if( timeline.value() != 0 || activated || (isUsingPresentWindows() && isMotionManagerMovingWindows()) )
         {
         if( activated )
             timeline.addTime(time);
@@ -123,11 +128,17 @@ void DesktopGridEffect::prePaintScreen( ScreenPrePaintData& data, int time )
             else
                 hoverTimeline[i].removeTime(time);
             }
+        if( isUsingPresentWindows() )
+            {
+            QList<WindowMotionManager>::iterator i;
+            for( i = m_managers.begin(); i != m_managers.end(); ++i )
+                (*i).calculate( time );
+            }
         // PAINT_SCREEN_BACKGROUND_FIRST is needed because screen will be actually painted more than once,
         // so with normal screen painting second screen paint would erase parts of the first paint
-        if( timeline.value() != 0 )
+        if( timeline.value() != 0 || (isUsingPresentWindows() && isMotionManagerMovingWindows()) )
             data.mask |= PAINT_SCREEN_TRANSFORMED | PAINT_SCREEN_BACKGROUND_FIRST;
-        if( !activated && timeline.value() == 0 )
+        if( !activated && timeline.value() == 0 && !(isUsingPresentWindows() && isMotionManagerMovingWindows()) )
             finish();
         }
     effects->prePaintScreen( data, time );
@@ -135,7 +146,7 @@ void DesktopGridEffect::prePaintScreen( ScreenPrePaintData& data, int time )
 
 void DesktopGridEffect::paintScreen( int mask, QRegion region, ScreenPaintData& data )
     {
-    if( timeline.value() == 0 )
+    if( timeline.value() == 0 && !isUsingPresentWindows() )
         {
         effects->paintScreen( mask, region, data );
         return;
@@ -146,6 +157,18 @@ void DesktopGridEffect::paintScreen( int mask, QRegion region, ScreenPaintData&
         paintingDesktop = desktop;
         effects->paintScreen( mask, region, d );
         }
+    if( isUsingPresentWindows() && windowMove )
+        {
+        // the moving window has to be painted on top of all desktops
+        QPoint diff = cursorPos() - m_windowMoveStartPoint;
+        QRect geo = m_windowMoveGeometry.translated( diff );
+        WindowPaintData d( windowMove );
+        d.xScale *= (float)geo.width()/(float)windowMove->width();
+        d.yScale *= (float)geo.height()/(float)windowMove->height();
+        d.xTranslate += qRound( geo.left() - windowMove->x() );
+        d.yTranslate += qRound( geo.top() - windowMove->y() );
+        effects->drawWindow( windowMove, PAINT_WINDOW_TRANSFORMED, infiniteRegion(), d );
+        }
 
     if( desktopNameAlignment )
         {
@@ -184,6 +207,8 @@ void DesktopGridEffect::postPaintScreen()
     {
     if( activated ? timeline.value() != 1 : timeline.value() != 0 )
         effects->addRepaintFull(); // Repaint during zoom
+    if( isUsingPresentWindows() && isMotionManagerMovingWindows() )
+        effects->addRepaintFull();
     if( activated )
         {
         for( int i = 0; i < effects->numberOfDesktops(); i++ )
@@ -203,11 +228,13 @@ void DesktopGridEffect::postPaintScreen()
 
 void DesktopGridEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData& data, int time )
     {
-    if( timeline.value() != 0 )
+    if( timeline.value() != 0 || (isUsingPresentWindows() && isMotionManagerMovingWindows()) )
         {
         if( w->isOnDesktop( paintingDesktop ))
             {
             w->enablePainting( EffectWindow::PAINT_DISABLED_BY_DESKTOP );
+            if( w->isMinimized() && isUsingPresentWindows() )
+                w->enablePainting( EffectWindow::PAINT_DISABLED_BY_MINIMIZE );
             data.mask |= PAINT_WINDOW_TRANSFORMED;
 
             // Split windows at screen edges
@@ -232,15 +259,20 @@ void DesktopGridEffect::prePaintWindow( EffectWindow* w, WindowPrePaintData& dat
 
 void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region, WindowPaintData& data )
     {
-    if( timeline.value() != 0 )
+    if( timeline.value() != 0 || (isUsingPresentWindows() && isMotionManagerMovingWindows()) )
         {
+        if( isUsingPresentWindows() && w == windowMove )
+            {
+            return; // will be painted on top of all other windows
+            }
+
         double xScale = data.xScale;
         double yScale = data.yScale;
 
         // Don't change brightness of windows on all desktops as this causes flickering
         if( !w->isOnAllDesktops() || w->isDesktop() )
             data.brightness *= 1.0 - ( 0.3 * ( 1.0 - hoverTimeline[paintingDesktop - 1].value() ));
-        
+
         for( int screen = 0; screen < effects->numScreens(); screen++ )
             {
             // Assume desktop windows can never be on two screens at once (Plasma makes one window per screen)
@@ -248,29 +280,57 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
                 screen = w->screen();
             QRect screenGeom = effects->clientArea( ScreenArea, screen, 0 );
 
+            QRectF transformedGeo = w->geometry();
             // Display all quads on the same screen on the same pass
             WindowQuadList screenQuads;
-            foreach( const WindowQuad &quad, data.quads )
+            bool quadsAdded = false;
+            if( isUsingPresentWindows() )
                 {
-                QRect quadRect(
-                    w->x() + quad.left(), w->y() + quad.top(),
-                    quad.right() - quad.left(), quad.bottom() - quad.top()
-                    );
-                if( quadRect.intersects( screenGeom ))
-                    screenQuads.append( quad );
+                WindowMotionManager& manager = m_managers[ (paintingDesktop-1)*(effects->numScreens())+screen ];
+                if( manager.isManaging( w ) )
+                    {
+                    foreach( const WindowQuad &quad, data.quads )
+                        screenQuads.append( quad );
+                    transformedGeo = manager.transformedGeometry( w );
+                    quadsAdded = true;
+                    }
+                else if( w->screen() != screen )
+                    quadsAdded = true; // we don't want parts of overlapping windows on the other screen
+                }
+            if( !quadsAdded )
+                {
+                foreach( const WindowQuad &quad, data.quads )
+                    {
+                    QRect quadRect(
+                        w->x() + quad.left(), w->y() + quad.top(),
+                        quad.right() - quad.left(), quad.bottom() - quad.top()
+                        );
+                    if( quadRect.intersects( screenGeom ) )
+                        screenQuads.append( quad );
+                    }
                 }
             if( screenQuads.isEmpty() )
                 continue; // Nothing is being displayed, don't bother
             WindowPaintData d = data;
             d.quads = screenQuads;
 
-            QPointF newPos = scalePos( QPoint( w->x(), w->y() ), paintingDesktop, screen);
+            QPointF newPos = scalePos( transformedGeo.topLeft().toPoint(), paintingDesktop, screen);
             double progress = timeline.value();
-            d.xScale = interpolate( 1, xScale * scale[screen], progress);
-            d.yScale = interpolate( 1, yScale * scale[screen], progress);
+            d.xScale = interpolate( 1, xScale * scale[screen] * (float)transformedGeo.width()/(float)w->geometry().width(), progress);
+            d.yScale = interpolate( 1, yScale * scale[screen] * (float)transformedGeo.height()/(float)w->geometry().height(), progress);
             d.xTranslate += qRound( newPos.x() - w->x() );
             d.yTranslate += qRound( newPos.y() - w->y() );
 
+            if( isUsingPresentWindows() && w->isDock() )
+                {
+                // fade out panels if present windows is used
+                d.opacity *= ( 1.0 - timeline.value() );
+                }
+            if( isUsingPresentWindows() && w->isMinimized() )
+                {
+                d.opacity *= timeline.value();
+                }
+
             if( effects->compositingType() == XRenderCompositing )
                 { // More exact clipping as XRender displays the entire window instead of just the quad
                 QPointF screenPosF = scalePos( screenGeom.topLeft(), paintingDesktop ).toPoint();
@@ -302,13 +362,61 @@ void DesktopGridEffect::paintWindow( EffectWindow* w, int mask, QRegion region,
 //-----------------------------------------------------------------------------
 // User interaction
 
+void DesktopGridEffect::windowAdded( EffectWindow* w )
+    {
+    if( !activated )
+        return;
+    if( isUsingPresentWindows() )
+        {
+        if( w->isDesktop() || w->isDock() || !w->visibleInClientGroup() )
+            return; // don't add
+        if( w->isOnAllDesktops() )
+            {
+            for( int i=0; i<effects->numberOfDesktops(); i++ )
+                {
+                WindowMotionManager& manager = m_managers[ i*effects->numScreens()+w->screen() ];
+                manager.manage( w );
+                m_proxy->calculateWindowTransformations( manager.managedWindows(), w->screen(), manager );
+                }
+            }
+        else
+            {
+            WindowMotionManager& manager = m_managers[ (w->desktop()-1)*effects->numScreens()+w->screen() ];
+            manager.manage( w );
+            m_proxy->calculateWindowTransformations( manager.managedWindows(), w->screen(), manager );
+            }
+        }
+    effects->addRepaintFull();
+    }
+
 void DesktopGridEffect::windowClosed( EffectWindow* w )
     {
+    if ( !activated )
+        return;
     if( w == windowMove )
         {
         effects->setElevatedWindow( windowMove, false );
         windowMove = NULL;
         }
+    if( isUsingPresentWindows() )
+        {
+        if( w->isOnAllDesktops() )
+            {
+            for( int i=0; i<effects->numberOfDesktops(); i++ )
+                {
+                WindowMotionManager& manager = m_managers[i*effects->numScreens()+w->screen()];
+                manager.unmanage(w);
+                m_proxy->calculateWindowTransformations(manager.managedWindows(), w->screen(), manager);
+                }
+            }
+        else
+            {
+            WindowMotionManager& manager = m_managers[(w->desktop()-1)*effects->numScreens()+w->screen()];
+            manager.unmanage(w);
+            m_proxy->calculateWindowTransformations(manager.managedWindows(), w->screen(), manager);
+            }
+        }
+    effects->addRepaintFull();
     }
 
 void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
@@ -322,29 +430,124 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
     if( e->type() == QEvent::MouseMove )
         {
         int d = posToDesktop( me->pos());
-        if( windowMove != NULL )
+        if( windowMove != NULL &&
+            (me->pos() - dragStartPos).manhattanLength() > KGlobalSettings::dndEventDelay() )
             { // Handle window moving
             if( !wasWindowMove ) // Activate on move
-                effects->activateWindow( windowMove );
+                {
+                if( isUsingPresentWindows() && windowMove->isOnAllDesktops() )
+                    {
+                    for( int i=0; i<effects->numberOfDesktops(); ++i )
+                        {
+                        WindowMotionManager& manager = m_managers[ (i)*(effects->numScreens()) + windowMove->screen() ];
+                        if( (i+1) == d )
+                            {
+                            const QRectF transformedGeo = manager.transformedGeometry( windowMove );
+                            const QPointF pos = scalePos( transformedGeo.topLeft().toPoint(), d, windowMove->screen() );
+                            const QSize size( scale[windowMove->screen()] * (float)transformedGeo.width(),
+                                                scale[windowMove->screen()] * (float)transformedGeo.height() );
+                            m_windowMoveGeometry = QRect( pos.toPoint(), size );
+                            m_windowMoveStartPoint = me->pos();
+                            }
+                        manager.unmanage( windowMove );
+                        m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
+                        }
+                    }
+                else if( isUsingPresentWindows() )
+                    {
+                    WindowMotionManager& manager = m_managers[ (windowMove->desktop()-1)*(effects->numScreens()) + windowMove->screen() ];
+                    const QRectF transformedGeo = manager.transformedGeometry( windowMove );
+                    const QPointF pos = scalePos( transformedGeo.topLeft().toPoint(), windowMove->desktop(), windowMove->screen() );
+                    const QSize size( scale[windowMove->screen()] * (float)transformedGeo.width(),
+                                        scale[windowMove->screen()] * (float)transformedGeo.height() );
+                    m_windowMoveGeometry = QRect( pos.toPoint(), size );
+                    m_windowMoveStartPoint = me->pos();
+
+                    manager.unmanage( windowMove );
+                    m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
+                    }
+                XDefineCursor( display(), input, QCursor( Qt::ClosedHandCursor ).handle() );
+                }
             wasWindowMove = true;
-            if( windowMove->isMovable() )
-                effects->moveWindow( windowMove, unscalePos( me->pos(), NULL ) + windowMoveDiff );
-            // TODO: Window snap
-            if( d != highlightedDesktop && !windowMove->isOnAllDesktops() )
-                effects->windowToDesktop( windowMove, d ); // Not true all desktop move
+            if( windowMove->isMovable() && !isUsingPresentWindows() )
+                {
+                int screen = effects->screenNumber( me->pos() );
+                effects->moveWindow( windowMove, unscalePos( me->pos(), NULL ) + windowMoveDiff, true, 1.0 / scale[screen] );
+                }
+            if( d != highlightedDesktop )
+                {
+                if( !windowMove->isOnAllDesktops() )
+                    effects->windowToDesktop( windowMove, d ); // Not true all desktop move
+                const int screen = effects->screenNumber( me->pos() );
+                if( screen != windowMove->screen() )
+                    effects->windowToScreen( windowMove, screen );
+                }
+            effects->addRepaintFull();
+            }
+        else if( (me->buttons() & Qt::LeftButton) && !wasDesktopMove &&
+            (me->pos() - dragStartPos).manhattanLength() > KGlobalSettings::dndEventDelay())
+            {
+            wasDesktopMove = true;
+            XDefineCursor( display(), input, QCursor( Qt::ClosedHandCursor ).handle() );
             }
         if( d != highlightedDesktop ) // Highlight desktop
+            {
+            if ( (me->buttons() & Qt::LeftButton) && !wasWindowMove )
+                {
+                EffectWindowList windows = effects->stackingOrder();
+                EffectWindowList stack;
+                foreach( EffectWindow* w, windows )
+                    {
+                    if( w->isOnAllDesktops() )
+                        continue;
+                    if( w->isOnDesktop( highlightedDesktop ) )
+                        {
+                        effects->windowToDesktop( w, d );
+                        if( isUsingPresentWindows() )
+                            {
+                            m_managers[ (d-1)*(effects->numScreens()) + w->screen() ].manage( w );
+                            m_managers[ (highlightedDesktop-1)*(effects->numScreens()) + w->screen() ].unmanage( w );
+                            }
+                        }
+                    else if( w->isOnDesktop( d ) )
+                        stack << w;
+                    }
+                foreach( EffectWindow* w, stack )
+                    {
+                    effects->windowToDesktop( w, highlightedDesktop );
+                    if( isUsingPresentWindows() )
+                        {
+                        m_managers[ (d-1)*(effects->numScreens()) + w->screen() ].unmanage( w );
+                        m_managers[ (highlightedDesktop-1)*(effects->numScreens()) + w->screen() ].manage( w );
+                        }
+                    }
+                if( isUsingPresentWindows() )
+                    {
+                    for( int i=0; i<effects->numScreens(); i++ )
+                        {
+                        WindowMotionManager& manager = m_managers[ (d-1)*(effects->numScreens()) + i ];
+                        WindowMotionManager& manager2 = m_managers[ (highlightedDesktop-1)*(effects->numScreens()) + i ];
+                        m_proxy->calculateWindowTransformations( manager.managedWindows(), i, manager );
+                        m_proxy->calculateWindowTransformations( manager2.managedWindows(), i, manager2 );
+                        }
+                    effects->addRepaintFull();
+                    }
+                }
             setHighlightedDesktop( d );
+            }
         }
     if( e->type() == QEvent::MouseButtonPress )
         {
         if( me->buttons() == Qt::LeftButton )
             {
-            QRect rect;
-            EffectWindow* w = windowAt( me->pos());
-            if( w != NULL && (w->isMovable() || w->isMovableAcrossScreens()) )
+//             QRect rect;
+            dragStartPos = me->pos();
+            bool isDesktop = (me->modifiers() & Qt::ControlModifier);
+            EffectWindow* w = isDesktop ? NULL : windowAt( me->pos());
+            if ( w != NULL )
+                isDesktop = w->isDesktop();
+            if( w != NULL && ( w->isMovable() || w->isMovableAcrossScreens() ))
                 { // Prepare it for moving
-                XDefineCursor( display(), input, QCursor( Qt::SizeAllCursor ).handle() );
                 windowMoveDiff = w->pos() - unscalePos( me->pos(), NULL );
                 windowMove = w;
                 effects->setElevatedWindow( windowMove, true );
@@ -356,29 +559,84 @@ void DesktopGridEffect::windowInputMouseEvent( Window, QEvent* e )
             if( w != NULL )
                 {
                 if( w->isOnAllDesktops())
-                    effects->windowToDesktop( w, posToDesktop( me->pos()));
+                    {
+                    const int desktop = posToDesktop( me->pos() );
+                    effects->windowToDesktop( w, desktop );
+                    if( isUsingPresentWindows() )
+                        {
+                        for( int i=0; i<effects->numberOfDesktops(); i++ )
+                            {
+                            if( i != desktop - 1 )
+                                {
+                                WindowMotionManager& manager = m_managers[ i*effects->numScreens() + w->screen() ];
+                                manager.unmanage( w );
+                                m_proxy->calculateWindowTransformations( manager.managedWindows(), w->screen(), manager );
+                                }
+                            }
+                        }
+                    }
                 else
+                    {
+                    if( isUsingPresentWindows() )
+                        {
+                        const int desktop = w->desktop();
+                        for( int i=0; i<effects->numberOfDesktops(); i++ )
+                            {
+                            if( i != desktop - 1 )
+                                {
+                                WindowMotionManager& manager = m_managers[ i*effects->numScreens() + w->screen() ];
+                                manager.manage( w );
+                                m_proxy->calculateWindowTransformations( manager.managedWindows(), w->screen(), manager );
+                                }
+                            }
+                        }
                     effects->windowToDesktop( w, NET::OnAllDesktops );
+                    }
                 effects->addRepaintFull();
                 }
             }
         }
     if( e->type() == QEvent::MouseButtonRelease && me->button() == Qt::LeftButton )
         {
-        if( !wasWindowMove )
+        if( !wasWindowMove && !wasDesktopMove )
             {
             setCurrentDesktop( posToDesktop( me->pos() ));
+            if( windowMove )
+                effects->activateWindow( windowMove );
             setActive( false );
             }
         if( windowMove )
             {
             if( wasWindowMove )
-                effects->activateWindow( windowMove ); // Just in case it was deactivated
+                {
+                if( isUsingPresentWindows() )
+                    {
+                    if( windowMove->isOnAllDesktops() )
+                        {
+                        for( int i=0; i<effects->numberOfDesktops(); ++i )
+                            {
+                            WindowMotionManager& manager = m_managers[ (i)*(effects->numScreens()) + windowMove->screen() ];
+                            manager.manage( windowMove );
+                            m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
+                            }
+                        }
+                    else
+                        {
+                        WindowMotionManager& manager = m_managers[ (windowMove->desktop()-1)*(effects->numScreens()) + windowMove->screen() ];
+                        manager.manage( windowMove );
+                        m_proxy->calculateWindowTransformations( manager.managedWindows(), windowMove->screen(), manager );
+                        }
+                    effects->addRepaintFull();
+                    }
+                }
             effects->setElevatedWindow( windowMove, false );
             windowMove = NULL;
             XDefineCursor( display(), input, QCursor( Qt::PointingHandCursor ).handle() );
             }
+        else if ( wasDesktopMove )
+            XDefineCursor( display(), input, QCursor( Qt::PointingHandCursor ).handle() );
         wasWindowMove = false;
+        wasDesktopMove = false;
         }
     }
 
@@ -584,9 +842,29 @@ EffectWindow* DesktopGridEffect::windowAt( QPoint pos ) const
 
     int desktop;
     pos = unscalePos( pos, &desktop );
-    foreach( EffectWindow* w, windows )
-        if( w->isOnDesktop( desktop ) && !w->isMinimized() && w->geometry().contains( pos ))
+    if( desktop > effects->numberOfDesktops() )
+        return NULL;
+    if( isUsingPresentWindows() )
+        {
+        const int screen = effects->screenNumber( pos );
+        EffectWindow *w =
+            m_managers.at((desktop-1)*(effects->numScreens())+screen).windowAtPoint( pos, false );
+        if( w )
             return w;
+        foreach( EffectWindow* w, windows )
+            {
+            if( w->isOnDesktop( desktop ) && w->isDesktop() && w->geometry().contains( pos ))
+                return w;
+            }
+        }
+    else
+        {
+        foreach( EffectWindow* w, windows )
+            {
+            if( w->isOnDesktop( desktop ) && !w->isMinimized() && w->geometry().contains( pos ))
+                return w;
+            }
+        }
     return NULL;
     }
 
@@ -744,23 +1022,29 @@ void DesktopGridEffect::setActive( bool active )
         return; // Only one fullscreen effect at a time thanks
     if( effects->numberOfDesktops() < 2 )
         return; // No point if there is only one desktop
+    if( active && isMotionManagerMovingWindows() )
+        return; // Still moving windows from last usage - don't activate
     if( activated == active )
         return; // Already in that state
 
-    // Example proxy code, TODO: Use or remove
-    //const PresentWindowsEffectProxy* proxy =
-    //    static_cast<const PresentWindowsEffectProxy*>( effects->getProxy( "presentwindows" ));
-    //if( proxy )
-    //    kDebug() << "Retrieved PresentWindowsEffectProxy, is present windows activate?"
-    //             << proxy->isActive();
-    //else
-    //    kDebug() << "Failed to retrieve PresentWindowsEffectProxy. Maybe present windows isn't enabled?";
-
     activated = active;
     if( activated && timeline.value() == 0 )
         setup();
     if( !activated )
+        {
+        if( isUsingPresentWindows() )
+            {
+            QList<WindowMotionManager>::iterator it;
+            for( it = m_managers.begin(); it != m_managers.end(); ++it )
+                {
+                foreach( EffectWindow* w, (*it).managedWindows() )
+                    {
+                    (*it).moveWindow( w, w->geometry() );
+                    }
+                }
+            }
         setHighlightedDesktop( effects->currentDesktop() ); // Ensure selected desktop is highlighted
+        }
     effects->addRepaintFull();
     }
 
@@ -850,6 +1134,30 @@ void DesktopGridEffect::setup()
         scaledSize.append( size );
         scaledOffset.append( offset );
         }
+
+    // setup the motion managers
+    if( m_usePresentWindows )
+        m_proxy = static_cast<PresentWindowsEffectProxy*>( effects->getProxy( "presentwindows" ) );
+    if( isUsingPresentWindows() )
+        {
+        for( int i=1; i<=effects->numberOfDesktops(); i++ )
+            {
+            for( int j=0; j<effects->numScreens(); j++ )
+                {
+                WindowMotionManager manager;
+                foreach( EffectWindow* w, effects->stackingOrder() )
+                    {
+                    if( w->isOnDesktop( i ) && w->screen() == j && !w->isDesktop() && !w->isDock() &&
+                        w->visibleInClientGroup() )
+                        {
+                        manager.manage( w );
+                        }
+                    }
+                m_proxy->calculateWindowTransformations(manager.managedWindows(), j, manager);
+                m_managers.append(manager);
+                }
+            }
+        }
     }
 
 void DesktopGridEffect::finish()
@@ -866,6 +1174,15 @@ void DesktopGridEffect::finish()
     keyboardGrab = false;
     effects->destroyInputWindow( input );
     effects->setActiveFullScreenEffect( 0 );
+    if( isUsingPresentWindows() )
+        {
+        while( !m_managers.isEmpty() )
+            {
+            m_managers.first().unmanageAll();
+            m_managers.removeFirst();
+            }
+        m_proxy = 0;
+        }
     }
 
 void DesktopGridEffect::globalShortcutChanged( const QKeySequence& seq )
@@ -873,6 +1190,25 @@ void DesktopGridEffect::globalShortcutChanged( const QKeySequence& seq )
     shortcut = KShortcut( seq );
     }
 
+bool DesktopGridEffect::isMotionManagerMovingWindows()
+    {
+    if( isUsingPresentWindows() )
+        {
+        QList<WindowMotionManager>::iterator it;
+        for( it = m_managers.begin(); it != m_managers.end(); ++it )
+            {
+            if( (*it).areWindowsMoving() )
+                return true;
+            }
+        }
+    return false;
+    }
+
+bool DesktopGridEffect::isUsingPresentWindows() const
+    {
+    return (m_proxy != NULL);
+    }
+
 } // namespace
 
 #include "desktopgrid.moc"
