@@ -33,7 +33,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #endif
 
 #include <QMouseEvent>
-#include <QPainter>
 
 #include <math.h>
 #include <assert.h>
@@ -47,14 +46,25 @@ KWIN_EFFECT( presentwindows, PresentWindowsEffect )
 PresentWindowsEffect::PresentWindowsEffect()
     : m_proxy( this )
     , m_activated( false )
-    , m_allDesktops( false )
     , m_ignoreMinimized( false )
     , m_decalOpacity( 0.0 )
     , m_hasKeyboardGrab( false )
     , m_tabBoxEnabled( false )
+    , m_mode( ModeCurrentDesktop )
+    , m_managerWindow( NULL )
     , m_highlightedWindow( NULL )
     , m_filterFrame( EffectFrame::Styled, false )
     {
+    m_atomDesktop = XInternAtom( display(), "_KDE_PRESENT_WINDOWS_DESKTOP", False );
+    m_atomWindows = XInternAtom( display(), "_KDE_PRESENT_WINDOWS_GROUP", False );
+    effects->registerPropertyType( m_atomDesktop, true );
+    effects->registerPropertyType( m_atomWindows, true );
+
+    // Announce support by creating a dummy version on the root window
+    unsigned char dummy = 0;
+    XChangeProperty( display(), rootWindow(), m_atomDesktop, m_atomDesktop, 8, PropModeReplace, &dummy, 1 );
+    XChangeProperty( display(), rootWindow(), m_atomWindows, m_atomWindows, 8, PropModeReplace, &dummy, 1 );
+
     QFont font;
     font.setPointSize( font.pointSize() * 2 );
     font.setBold( true );
@@ -73,11 +83,21 @@ PresentWindowsEffect::PresentWindowsEffect()
     shortcutAll = b->globalShortcut();
     connect( b, SIGNAL( triggered(bool) ), this, SLOT( toggleActiveAllDesktops() ));
     connect( b, SIGNAL( globalShortcutChanged(QKeySequence) ), this, SLOT( globalShortcutChangedAll(QKeySequence)));
+    KAction* c = ( KAction* )actionCollection->addAction( "ExposeClass" );
+    c->setText( i18n( "Toggle Present Windows (Window class)" ));
+    c->setGlobalShortcut( KShortcut( Qt::CTRL + Qt::Key_F7 ));
+    connect( c, SIGNAL( triggered(bool) ), this, SLOT( toggleActiveClass() ));
+    connect( c, SIGNAL( globalShortcutChanged(QKeySequence) ), this, SLOT( globalShortcutChangedClass(QKeySequence)));
+    shortcutClass = c->globalShortcut();
     reconfigure( ReconfigureAll );
     }
 
 PresentWindowsEffect::~PresentWindowsEffect()
     {
+    XDeleteProperty( display(), rootWindow(), m_atomDesktop );
+    effects->registerPropertyType( m_atomDesktop, false );
+    XDeleteProperty( display(), rootWindow(), m_atomWindows );
+    effects->registerPropertyType( m_atomWindows, false );
     foreach( ElectricBorder border, m_borderActivate )
         {
         effects->unreserveElectricBorder( border );
@@ -121,18 +141,37 @@ void PresentWindowsEffect::reconfigure( ReconfigureFlags )
     m_showCaptions = conf.readEntry( "DrawWindowCaptions", true );
     m_showIcons = conf.readEntry( "DrawWindowIcons", true );
     m_tabBoxAllowed = conf.readEntry( "TabBox", false );
+    m_tabBoxAlternativeAllowed = conf.readEntry( "TabBoxAlternative", false );
     m_ignoreMinimized = conf.readEntry( "IgnoreMinimized", false );
     m_accuracy = conf.readEntry( "Accuracy", 1 ) * 20;
     m_fillGaps = conf.readEntry( "FillGaps", true );
     m_fadeDuration = double( animationTime( 150 ));
     m_showPanel = conf.readEntry( "ShowPanel", false );
+    m_leftButtonWindow = (WindowMouseAction)conf.readEntry( "LeftButtonWindow", (int)WindowActivateAction );
+    m_middleButtonWindow = (WindowMouseAction)conf.readEntry( "MiddleButtonWindow", (int)WindowNoAction );
+    m_rightButtonWindow = (WindowMouseAction)conf.readEntry( "RightButtonWindow", (int)WindowExitAction );
+    m_leftButtonDesktop = (DesktopMouseAction)conf.readEntry( "LeftButtonDesktop", (int)DesktopExitAction );
+    m_middleButtonDesktop = (DesktopMouseAction)conf.readEntry( "MiddleButtonDesktop", (int)DesktopNoAction );
+    m_rightButtonDesktop = (DesktopMouseAction)conf.readEntry( "RightButtonDesktop", (int)DesktopNoAction );
     }
 
-const void* PresentWindowsEffect::proxy() const
+void* PresentWindowsEffect::proxy()
     {
     return &m_proxy;
     }
 
+void PresentWindowsEffect::toggleActiveClass()
+{
+    if( !m_activated )
+        {
+        if( !effects->activeWindow() )
+            return;
+        m_mode = ModeWindowClass;
+        m_class = effects->activeWindow()->windowClass();
+        }
+    setActive( !m_activated );
+}
+
 //-----------------------------------------------------------------------------
 // Screen painting
 
@@ -182,14 +221,15 @@ void PresentWindowsEffect::postPaintScreen()
         }
 
     // Update windows that are changing brightness or opacity
-    foreach( EffectWindow *w, m_windowData.keys() )
+    DataHash::const_iterator i;
+    for( i = m_windowData.constBegin(); i != m_windowData.constEnd(); ++i )
         {
-        if( m_windowData[w].opacity > 0.0 && m_windowData[w].opacity < 1.0 )
-            w->addRepaintFull();
-        if( m_windowData[w].highlight > 0.0 && m_windowData[w].highlight < 1.0 )
-            w->addRepaintFull();
+        if( i.value().opacity > 0.0 && i.value().opacity < 1.0 )
+            i.key()->addRepaintFull();
+        if( i.value().highlight > 0.0 && i.value().highlight < 1.0 )
+            i.key()->addRepaintFull();
         }
-    
+
     effects->postPaintScreen();
     }
 
@@ -204,6 +244,8 @@ void PresentWindowsEffect::prePaintWindow( EffectWindow *w, WindowPrePaintData &
         {
         w->enablePainting( EffectWindow::PAINT_DISABLED_BY_MINIMIZE ); // Display always
         w->enablePainting( EffectWindow::PAINT_DISABLED_BY_DESKTOP );
+        if( m_windowData[w].visible )
+            w->enablePainting( EffectWindow::PAINT_DISABLED_BY_CLIENT_GROUP );
 
         // Calculate window's opacity
         // TODO: Minimized windows or windows not on the current desktop are only 75% visible?
@@ -247,6 +289,10 @@ void PresentWindowsEffect::prePaintWindow( EffectWindow *w, WindowPrePaintData &
                 w->enablePainting( EffectWindow::PAINT_DISABLED_BY_DELETE );
             }
 
+        // desktop windows on other desktops (Plasma activity per desktop) should not be painted
+        if( w->isDesktop() && !w->isOnCurrentDesktop() )
+            w->disablePainting( EffectWindow::PAINT_DISABLED_BY_DESKTOP );
+
         if( m_motionManager.isManaging( w ))
             data.setTransformed(); // We will be moving this window
         }
@@ -340,6 +386,8 @@ void PresentWindowsEffect::windowAdded( EffectWindow *w )
 
 void PresentWindowsEffect::windowClosed( EffectWindow *w )
     {
+    if( m_managerWindow == w )
+        m_managerWindow = NULL;
     if( !m_windowData.contains( w ))
         return;
     m_windowData[w].deleted = true;
@@ -376,16 +424,19 @@ bool PresentWindowsEffect::borderActivated( ElectricBorder border )
 void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent *e )
     {
     assert( w == m_input );
+    Q_UNUSED( w );
 
     // Which window are we hovering over? Always trigger as we don't always get move events before clicking
     // We cannot use m_motionManager.windowAtPoint() as the window might not be visible
     EffectWindowList windows = m_motionManager.managedWindows();
+    bool hovering = false;
     for( int i = 0; i < windows.size(); i++ )
         {
         assert( m_windowData.contains( windows.at( i )));
         if( m_motionManager.transformedGeometry( windows.at( i )).contains( cursorPos() ) &&
             m_windowData[windows.at( i )].visible && !m_windowData[windows.at( i )].deleted )
             {
+            hovering = true;
             if( windows.at( i ) && m_highlightedWindow != windows.at( i ))
                 setHighlightedWindow( windows.at( i ));
             break;
@@ -398,55 +449,166 @@ void PresentWindowsEffect::windowInputMouseEvent( Window w, QEvent *e )
     QMouseEvent* me = static_cast<QMouseEvent*>( e );
     if( me->button() == Qt::LeftButton )
         {
-        if( m_highlightedWindow )
-            effects->activateWindow( m_highlightedWindow );
-        setActive( false );
+        if( hovering )
+            {
+            // mouse is hovering above a window - use MouseActionsWindow
+            mouseActionWindow( m_leftButtonWindow );
+            }
+        else
+            {
+            // mouse is hovering above desktop - use MouseActionsDesktop
+            mouseActionDesktop( m_leftButtonDesktop );
+            }
+        }
+    if( me->button() == Qt::MidButton )
+        {
+        if( hovering )
+            {
+            // mouse is hovering above a window - use MouseActionsWindow
+            mouseActionWindow( m_middleButtonWindow );
+            }
+        else
+            {
+            // mouse is hovering above desktop - use MouseActionsDesktop
+            mouseActionDesktop( m_middleButtonDesktop );
+            }
+        }
+    if( me->button() == Qt::RightButton )
+        {
+        if( hovering )
+            {
+            // mouse is hovering above a window - use MouseActionsWindow
+            mouseActionWindow( m_rightButtonWindow );
+            }
+        else
+            {
+            // mouse is hovering above desktop - use MouseActionsDesktop
+            mouseActionDesktop( m_rightButtonDesktop );
+            }
         }
-    // TODO: User mouse actions. E.g. right-click stickies and middle-click brings to current desktop
     }
 
+void PresentWindowsEffect::mouseActionWindow( WindowMouseAction& action )
+    {
+    switch( action )
+        {
+        case WindowActivateAction:
+            if( m_highlightedWindow )
+                effects->activateWindow( m_highlightedWindow );
+            setActive( false );
+            break;
+        case WindowExitAction:
+            setActive( false );
+            break;
+        case WindowToCurrentDesktopAction:
+            if( m_highlightedWindow )
+                effects->windowToDesktop( m_highlightedWindow, effects->currentDesktop() );
+            break;
+        case WindowToAllDesktopsAction:
+            if( m_highlightedWindow )
+                {
+                if( m_highlightedWindow->isOnAllDesktops() )
+                    effects->windowToDesktop( m_highlightedWindow, effects->currentDesktop() );
+                else
+                    effects->windowToDesktop( m_highlightedWindow, NET::OnAllDesktops );
+                }
+            break;
+        case WindowMinimizeAction:
+            if( m_highlightedWindow )
+                {
+                if( m_highlightedWindow->isMinimized() )
+                    m_highlightedWindow->unminimize();
+                else
+                    m_highlightedWindow->minimize();
+                }
+            break;
+        case WindowCloseAction:
+            if( m_highlightedWindow )
+                {
+                m_highlightedWindow->closeWindow();
+                }
+            break;
+        default:
+            break;
+        }
+    }
+
+void PresentWindowsEffect::mouseActionDesktop( DesktopMouseAction& action )
+    {
+    switch( action )
+        {
+        case DesktopActivateAction:
+            if( m_highlightedWindow )
+                effects->activateWindow( m_highlightedWindow );
+            setActive( false );
+            break;
+        case DesktopExitAction:
+            setActive( false );
+            break;
+        case DesktopShowDesktopAction:
+            effects->setShowingDesktop( true );
+            setActive( false );
+        default:
+            break;
+        }
+    }
+
+
 void PresentWindowsEffect::grabbedKeyboardEvent( QKeyEvent *e )
     {
     if( e->type() == QEvent::KeyPress )
         {
         // check for global shortcuts
         // HACK: keyboard grab disables the global shortcuts so we have to check for global shortcut (bug 156155)
-        if( !m_allDesktops && shortcut.contains( e->key() + e->modifiers() ) )
+        if( m_mode == ModeCurrentDesktop && shortcut.contains( e->key() + e->modifiers() ) )
             {
             toggleActive();
             return;
             }
-        if( m_allDesktops && shortcutAll.contains( e->key() + e->modifiers() ) )
+        if( m_mode == ModeAllDesktops && shortcutAll.contains( e->key() + e->modifiers() ) )
             {
             toggleActiveAllDesktops();
             return;
             }
+        if( m_mode == ModeWindowClass && shortcutClass.contains( e->key() + e->modifiers() ) )
+            {
+            toggleActiveClass();
+            return;
+            }
 
         switch( e->key() )
             { // Wrap only if not auto-repeating
             case Qt::Key_Left:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, -1, 0, !e->isAutoRepeat() ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, -1, 0, !e->isAutoRepeat() ));
                 break;
             case Qt::Key_Right:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, 1, 0, !e->isAutoRepeat() ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, 1, 0, !e->isAutoRepeat() ));
                 break;
             case Qt::Key_Up:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, -1, !e->isAutoRepeat() ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, -1, !e->isAutoRepeat() ));
                 break;
             case Qt::Key_Down:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, 1, !e->isAutoRepeat() ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, 1, !e->isAutoRepeat() ));
                 break;
             case Qt::Key_Home:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, -1000, 0, false ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, -1000, 0, false ));
                 break;
             case Qt::Key_End:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, 1000, 0, false ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, 1000, 0, false ));
                 break;
             case Qt::Key_PageUp:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, -1000, false ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, -1000, false ));
                 break;
             case Qt::Key_PageDown:
-                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, 1000, false ));
+                if( m_highlightedWindow )
+                    setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, 1000, false ));
                 break;
             case Qt::Key_Backspace:
                 if( !m_windowFilter.isEmpty() )
@@ -499,9 +661,9 @@ void PresentWindowsEffect::tabBoxAdded( int mode )
         return;
     if( m_activated )
         return;
-    if( !m_tabBoxAllowed )
-        return;
-    if( mode == TabBoxWindowsMode && effects->currentTabBoxWindowList().count() > 0 )
+    if( (( mode == TabBoxWindowsMode && m_tabBoxAllowed ) ||
+        ( mode == TabBoxWindowsAlternativeMode && m_tabBoxAlternativeAllowed )) &&
+        effects->currentTabBoxWindowList().count() > 0 )
         {
         m_tabBoxEnabled = true;
         setActive( true );
@@ -526,6 +688,127 @@ void PresentWindowsEffect::tabBoxUpdated()
         setHighlightedWindow( effects->currentTabBoxWindow() );
     }
 
+void PresentWindowsEffect::tabBoxKeyEvent( QKeyEvent* event )
+    {
+    if( !m_activated )
+        return;
+    // not using the "normal" grabbedKeyboardEvent as we don't want to filter in tabbox
+    if( event->type() == QEvent::KeyPress )
+        {
+        switch( event->key() )
+            { // Wrap only if not auto-repeating
+            case Qt::Key_Left:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, -1, 0, !event->isAutoRepeat() ));
+                break;
+            case Qt::Key_Right:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, 1, 0, !event->isAutoRepeat() ));
+                break;
+            case Qt::Key_Up:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, -1, !event->isAutoRepeat() ));
+                break;
+            case Qt::Key_Down:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, 1, !event->isAutoRepeat() ));
+                break;
+            case Qt::Key_Home:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, -1000, 0, false ));
+                break;
+            case Qt::Key_End:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, 1000, 0, false ));
+                break;
+            case Qt::Key_PageUp:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, -1000, false ));
+                break;
+            case Qt::Key_PageDown:
+                setHighlightedWindow( relativeWindow( m_highlightedWindow, 0, 1000, false ));
+                break;
+            default:
+                // nothing
+                break;
+            }
+        }
+    }
+
+//-----------------------------------------------------------------------------
+// Atom handling
+void PresentWindowsEffect::propertyNotify( EffectWindow* w, long a )
+    {
+    if( a != m_atomDesktop && a != m_atomWindows )
+        return; // Not our atom
+
+    if( a == m_atomDesktop )
+        {
+        QByteArray byteData = w->readProperty( m_atomDesktop, m_atomDesktop, 32 );
+        if( byteData.length() < 1 )
+            {
+            // Property was removed, end present windows
+            setActive( false );
+            return;
+            }
+        long* data = reinterpret_cast<long*>( byteData.data() );
+
+        if( !data[0] )
+            {
+            // Purposely ending present windows by issuing a NULL target
+            setActive( false );
+            return;
+            }
+        // present windows is active so don't do anything
+        if( m_activated )
+            return;
+
+        int desktop = data[0];
+        if( desktop > effects->numberOfDesktops() )
+            return;
+        if( desktop == -1 )
+            toggleActiveAllDesktops();
+        else
+            {
+            m_mode = ModeSelectedDesktop;
+            m_desktop = desktop;
+            m_managerWindow = w;
+            setActive( true );
+            }
+        }
+    else if( a == m_atomWindows )
+        {
+        QByteArray byteData = w->readProperty( m_atomWindows, m_atomWindows, 32 );
+        if( byteData.length() < 1 )
+            {
+            // Property was removed, end present windows
+            setActive( false );
+            return;
+            }
+        long* data = reinterpret_cast<long*>( byteData.data() );
+
+        if( !data[0] )
+            {
+            // Purposely ending present windows by issuing a NULL target
+            setActive( false );
+            return;
+            }
+        // present windows is active so don't do anything
+        if( m_activated )
+            return;
+
+        // for security clear selected windows
+        m_selectedWindows.clear();
+        int length = byteData.length() / sizeof( data[0] );
+        for( int i=0; i<length; i++ )
+            {
+            EffectWindow* foundWin = effects->findWindow( data[i] );
+            if( !foundWin )
+                {
+                kDebug(1212) << "Invalid window targetted for present windows. Requested:" << data[i];
+                continue;
+                }
+            m_selectedWindows.append( foundWin );
+            }
+        m_mode = ModeWindowGroup;
+        m_managerWindow = w;
+        setActive( true );
+        }
+    }
+
 //-----------------------------------------------------------------------------
 // Window rearranging
 
@@ -622,12 +905,7 @@ void PresentWindowsEffect::rearrangeWindows()
         if( !windows.count() )
             continue;
 
-        if( m_layoutMode == LayoutRegularGrid || m_tabBoxEnabled ) // Force the grid for window switching
-            calculateWindowTransformationsClosest( windows, screen );
-        else if( m_layoutMode == LayoutFlexibleGrid )
-            calculateWindowTransformationsKompose( windows, screen );
-        else
-            calculateWindowTransformationsNatural( windows, screen );
+        calculateWindowTransformations( windows, screen, m_motionManager );
         }
 
     // Resize text frames if required
@@ -644,8 +922,28 @@ void PresentWindowsEffect::rearrangeWindows()
     delete metrics;
     }
 
-void PresentWindowsEffect::calculateWindowTransformationsClosest( EffectWindowList windowlist, int screen )
+void PresentWindowsEffect::calculateWindowTransformations( EffectWindowList windowlist, int screen,
+        WindowMotionManager& motionManager, bool external )
     {
+    if( m_layoutMode == LayoutRegularGrid || m_tabBoxEnabled ) // Force the grid for window switching
+        calculateWindowTransformationsClosest( windowlist, screen, motionManager );
+    else if( m_layoutMode == LayoutFlexibleGrid )
+        calculateWindowTransformationsKompose( windowlist, screen, motionManager );
+    else
+        calculateWindowTransformationsNatural( windowlist, screen, motionManager );
+
+    // If called externally we don't need to remember this data
+    if( external )
+        m_windowData.clear();
+    }
+
+void PresentWindowsEffect::calculateWindowTransformationsClosest( EffectWindowList windowlist, int screen,
+        WindowMotionManager& motionManager )
+    {
+    // This layout mode requires at least one window visible
+    if( windowlist.count() == 0 )
+        return;
+
     QRect area = effects->clientArea( ScreenArea, screen, effects->currentDesktop() );
     if( m_showPanel ) // reserve space for the panel
         area = effects->clientArea( MaximizeArea, screen, effects->currentDesktop() );
@@ -653,8 +951,12 @@ void PresentWindowsEffect::calculateWindowTransformationsClosest( EffectWindowLi
     int rows = int( ceil( windowlist.count() / double( columns )));
 
     // Remember the size for later
-    m_gridSizes[screen].columns = columns;
-    m_gridSizes[screen].rows = rows;
+    // If we are using this layout externally we don't need to remember m_gridSizes.
+    if( m_gridSizes.size() != 0 )
+        {
+        m_gridSizes[screen].columns = columns;
+        m_gridSizes[screen].rows = rows;
+        }
 
     // Assign slots
     foreach( EffectWindow *w, windowlist )
@@ -723,12 +1025,17 @@ void PresentWindowsEffect::calculateWindowTransformationsClosest( EffectWindowLi
                 target.center().y() - int( w->height() * scale ) / 2,
                 scale * w->width(), scale * w->height() );
         }
-        m_motionManager.moveWindow( w, target );
+        motionManager.moveWindow( w, target );
         }
     }
 
-void PresentWindowsEffect::calculateWindowTransformationsKompose( EffectWindowList windowlist, int screen )
+void PresentWindowsEffect::calculateWindowTransformationsKompose( EffectWindowList windowlist, int screen,
+        WindowMotionManager& motionManager )
     {
+    // This layout mode requires at least one window visible
+    if( windowlist.count() == 0 )
+        return;
+
     QRect availRect = effects->clientArea( ScreenArea, screen, effects->currentDesktop() );
     if( m_showPanel ) // reserve space for the panel
         availRect = effects->clientArea( MaximizeArea, screen, effects->currentDesktop() );
@@ -862,7 +1169,7 @@ void PresentWindowsEffect::calculateWindowTransformationsKompose( EffectWindowLi
             QRect target = geometryRects[pos];
             target.setY( target.y() + topOffset );
             m_windowData[window].slot = pos;
-            m_motionManager.moveWindow( window, target );
+            motionManager.moveWindow( window, target );
 
             //kDebug(1212) << "Window '" << window->caption() << "' gets moved to (" <<
             //        mWindowData[window].area.left() << "; " << mWindowData[window].area.right() <<
@@ -873,20 +1180,24 @@ void PresentWindowsEffect::calculateWindowTransformationsKompose( EffectWindowLi
         }
     }
 
-void PresentWindowsEffect::calculateWindowTransformationsNatural( EffectWindowList windowlist, int screen )
+void PresentWindowsEffect::calculateWindowTransformationsNatural( EffectWindowList windowlist, int screen,
+        WindowMotionManager& motionManager )
     {
     // If windows do not overlap they scale into nothingness, fix by resetting. To reproduce
     // just have a single window on a Xinerama screen or have two windows that do not touch.
     // TODO: Work out why this happens, is most likely a bug in the manager.
     foreach( EffectWindow *w, windowlist )
-        if( m_motionManager.transformedGeometry( w ) == w->geometry() )
-            m_motionManager.reset( w );
+        if( motionManager.transformedGeometry( w ) == w->geometry() )
+            motionManager.reset( w );
 
     if( windowlist.count() == 1 )
         {
         // Just move the window to its original location to save time
-        m_motionManager.moveWindow( windowlist[0], windowlist[0]->geometry() );
-        return;
+        if( effects->clientArea( FullScreenArea, windowlist[0] ).contains( windowlist[0]->geometry() ) )
+            {
+            motionManager.moveWindow( windowlist[0], windowlist[0]->geometry() );
+            return;
+            }
         }
 
     // As we are using pseudo-random movement (See "slot") we need to make sure the list
@@ -899,13 +1210,14 @@ void PresentWindowsEffect::calculateWindowTransformationsNatural( EffectWindowLi
     QRect bounds = area;
     int direction = 0;
     QHash<EffectWindow*, QRect> targets;
+    QHash<EffectWindow*, int> directions;
     foreach( EffectWindow *w, windowlist )
         {
         bounds = bounds.united( w->geometry() );
         targets[w] = w->geometry();
         // Reuse the unused "slot" as a preferred direction attribute. This is used when the window
         // is on the edge of the screen to try to use as much screen real estate as possible.
-        m_windowData[w].slot = direction;
+        directions[w] = direction;
         direction++;
         if( direction == 4 )
             direction = 0;
@@ -958,9 +1270,9 @@ void PresentWindowsEffect::calculateWindowTransformationsNatural( EffectWindowLi
                     if( xSection != 1 || ySection != 1 ) // Remove this if you want the center to pull as well
                         {
                         if( xSection == 1 )
-                            xSection = ( m_windowData[w].slot / 2 ? 2 : 0 );
+                            xSection = ( directions[w] / 2 ? 2 : 0 );
                         if( ySection == 1 )
-                            ySection = ( m_windowData[w].slot % 2 ? 2 : 0 );
+                            ySection = ( directions[w] % 2 ? 2 : 0 );
                         }
                     if( xSection == 0 && ySection == 0 )
                         diff = QPoint( bounds.topLeft() - targets[w].center() );
@@ -1104,7 +1416,7 @@ void PresentWindowsEffect::calculateWindowTransformationsNatural( EffectWindowLi
 
     // Notify the motion manager of the targets
     foreach( EffectWindow *w, windowlist )
-        m_motionManager.moveWindow( w, targets[w] );
+        motionManager.moveWindow( w, targets[w] );
     }
 
 //-----------------------------------------------------------------------------
@@ -1173,12 +1485,13 @@ bool PresentWindowsEffect::isOverlappingAny( EffectWindow *w, const QHash<Effect
     if( border.intersects( targets[w] ))
         return true;
     // Is there a better way to do this?
-    foreach( EffectWindow *e, targets.keys() )
+    QHash<EffectWindow*, QRect>::const_iterator i;
+    for( i = targets.constBegin(); i != targets.constEnd(); ++i )
         {
-        if( w == e )
+        if( w == i.key() )
             continue;
         if( targets[w].adjusted( -5, -5, 5, 5 ).intersects(
-            targets[e].adjusted( -5, -5, 5, 5 )))
+            i.value().adjusted( -5, -5, 5, 5 )))
             return true;
         }
     return false;
@@ -1286,18 +1599,17 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
     else
         {
         // Fade in/out all windows
+        EffectWindow *activeWindow = effects->activeWindow();
+        if( m_tabBoxEnabled )
+            activeWindow = effects->currentTabBoxWindow();
+        int desktop = effects->currentDesktop();
+        if( activeWindow && !activeWindow->isOnAllDesktops() )
+            desktop = activeWindow->desktop();
         foreach( EffectWindow *w, effects->stackingOrder() )
             {
             assert( m_windowData.contains( w ));
-            EffectWindow *activeWindow = effects->activeWindow();
-            if( m_tabBoxEnabled )
-                activeWindow = effects->currentTabBoxWindow();
-            if( activeWindow )
-                m_windowData[w].visible = ( w->desktop() == activeWindow->desktop() || w->isOnAllDesktops() ) && !w->isMinimized();
-            else // Deactivating to an empty desktop
-                m_windowData[w].visible = ( w->isOnCurrentDesktop() || w->isOnAllDesktops() ) && !w->isMinimized();
-            if( m_tabBoxEnabled && w == effects->currentTabBoxWindow() )
-                m_windowData[w].visible = true;
+            m_windowData[w].visible = ( w->isOnDesktop( desktop ) || w->isOnAllDesktops() ) &&
+                !w->isMinimized() && ( w->visibleInClientGroup() || m_windowData[w].visible );
             }
 
         // Move all windows back to their original position
@@ -1305,11 +1617,22 @@ void PresentWindowsEffect::setActive( bool active, bool closingTab )
             m_motionManager.moveWindow( w, w->geometry() );
         m_filterFrame.free();
         m_windowFilter.clear();
+        m_selectedWindows.clear();
 
         effects->destroyInputWindow( m_input );
         if( m_hasKeyboardGrab )
             effects->ungrabKeyboard();
         m_hasKeyboardGrab = false;
+
+        // destroy atom on manager window
+        if( m_managerWindow )
+            {
+            if( m_mode == ModeSelectedDesktop )
+                m_managerWindow->deleteProperty( m_atomDesktop );
+            else if( m_mode == ModeWindowGroup )
+                m_managerWindow->deleteProperty( m_atomWindows );
+            m_managerWindow = NULL;
+            }
         }
     effects->addRepaintFull(); // Trigger the first repaint
     }
@@ -1335,8 +1658,21 @@ bool PresentWindowsEffect::isSelectableWindow( EffectWindow *w )
         return false;
     if( !w->acceptsFocus() )
         return false;
-    if( !m_allDesktops && !w->isOnCurrentDesktop() )
+    if( !w->visibleInClientGroup() )
         return false;
+    switch( m_mode )
+        {
+        case ModeAllDesktops:
+            break;
+        case ModeCurrentDesktop:
+            return w->isOnCurrentDesktop();
+        case ModeSelectedDesktop:
+            return w->isOnDesktop( m_desktop );
+        case ModeWindowGroup:
+            return m_selectedWindows.contains( w );
+        case ModeWindowClass:
+            return m_class == w->windowClass();
+        }
     if( !m_tabBoxEnabled && m_ignoreMinimized && w->isMinimized() )
         return false;
     return true;
@@ -1558,6 +1894,11 @@ void PresentWindowsEffect::globalShortcutChangedAll( const QKeySequence& seq )
     shortcutAll = KShortcut( seq );
     }
 
+void PresentWindowsEffect::globalShortcutChangedClass( const QKeySequence& seq )
+    {
+    shortcutClass = KShortcut( seq );
+    }
+
 } // namespace
 
 #include "presentwindows.moc"
