@@ -56,13 +56,25 @@ KWinScreenEdgesConfig::KWinScreenEdgesConfig( QWidget* parent, const QVariantLis
     connect( m_ui->desktopSwitchCombo, SIGNAL( currentIndexChanged(int) ), this, SLOT( changed() ));
     connect( m_ui->activationDelaySpin, SIGNAL( valueChanged(int) ), this, SLOT( changed() ));
     connect( m_ui->triggerCooldownSpin, SIGNAL( valueChanged(int) ), this, SLOT( changed() ));
+    connect( m_ui->quickMaximizeBox, SIGNAL( stateChanged(int) ), this, SLOT( changed() ));
+    connect( m_ui->quickTileBox, SIGNAL( stateChanged(int) ), this, SLOT( changed() ));
 
     // Visual feedback of action group conflicts
     connect( m_ui->desktopSwitchCombo, SIGNAL( currentIndexChanged(int) ), this, SLOT( groupChanged() ));
+    connect( m_ui->quickMaximizeBox, SIGNAL( stateChanged(int) ), this, SLOT( groupChanged() ));
+    connect( m_ui->quickTileBox, SIGNAL( stateChanged(int) ), this, SLOT( groupChanged() ));
 
-    if( CompositingPrefs::compositingPossible() )
+    // NOTICE: this is intended to workaround broken GL implementations that successfully segfault on glXQuery :-(
+    KConfigGroup gl_workaround_config(m_config, "Compositing");
+    const bool checkIsSafe = gl_workaround_config.readEntry("CheckIsSafe", true);
+    if( checkIsSafe && CompositingPrefs::compositingPossible() )
+        {
+        gl_workaround_config.writeEntry("CheckIsSafe", false);
+        gl_workaround_config.sync();
         m_defaultPrefs.detect(); // Driver-specific config detection
-
+        gl_workaround_config.writeEntry("CheckIsSafe", true);
+        gl_workaround_config.sync();
+        }
     load();
     }
 
@@ -80,10 +92,6 @@ void KWinScreenEdgesConfig::groupChanged()
     monitorHideEdge( ElectricRight, hide );
     monitorHideEdge( ElectricBottom, hide );
     monitorHideEdge( ElectricLeft, hide );
-
-    // Desktop switch conflicts
-    m_ui->desktopSwitchLabel->setEnabled( true );
-    m_ui->desktopSwitchCombo->setEnabled( true );
     }
 
 void KWinScreenEdgesConfig::load()
@@ -97,6 +105,8 @@ void KWinScreenEdgesConfig::load()
     m_ui->desktopSwitchCombo->setCurrentIndex( config.readEntry( "ElectricBorders", 0 ));
     m_ui->activationDelaySpin->setValue( config.readEntry( "ElectricBorderDelay", 150 ));
     m_ui->triggerCooldownSpin->setValue( config.readEntry( "ElectricBorderCooldown", 350 ));
+    m_ui->quickMaximizeBox->setChecked( config.readEntry( "ElectricBorderMaximize", true ));
+    m_ui->quickTileBox->setChecked( config.readEntry( "ElectricBorderTiling", true ));
 
     emit changed( false );
     }
@@ -112,6 +122,8 @@ void KWinScreenEdgesConfig::save()
     config.writeEntry( "ElectricBorders", m_ui->desktopSwitchCombo->currentIndex() );
     config.writeEntry( "ElectricBorderDelay", m_ui->activationDelaySpin->value() );
     config.writeEntry( "ElectricBorderCooldown", m_ui->triggerCooldownSpin->value() );
+    config.writeEntry( "ElectricBorderMaximize", m_ui->quickMaximizeBox->isChecked() );
+    config.writeEntry( "ElectricBorderTiling", m_ui->quickTileBox->isChecked() );
 
     config.sync();
 
@@ -129,6 +141,8 @@ void KWinScreenEdgesConfig::defaults()
     m_ui->desktopSwitchCombo->setCurrentIndex( 0 );
     m_ui->activationDelaySpin->setValue( 150 );
     m_ui->triggerCooldownSpin->setValue( 350 );
+    m_ui->quickMaximizeBox->setChecked( true );
+    m_ui->quickTileBox->setChecked( true );
 
     emit changed( true );
     }
@@ -144,7 +158,7 @@ void KWinScreenEdgesConfig::showEvent( QShowEvent* e )
 bool KWinScreenEdgesConfig::effectEnabled( const QString& effect, const KConfigGroup& cfg ) const
     {
     KService::List services = KServiceTypeTrader::self()->query(
-        "KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_" + effect + "'");
+        "KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_" + effect + '\'');
     if( services.isEmpty())
         return false;
     QVariant v = services.first()->property("X-KDE-PluginInfo-EnabledByDefault");
@@ -170,6 +184,7 @@ void KWinScreenEdgesConfig::monitorInit()
     {
     monitorAddItem( i18n( "No Action" ));
     monitorAddItem( i18n( "Show Dashboard" ));
+    monitorAddItem( i18n( "Show Desktop" ));
 
     // Search the effect names
     KServiceTypeTrader* trader = KServiceTypeTrader::self();
@@ -213,6 +228,19 @@ void KWinScreenEdgesConfig::monitorInit()
         monitorAddItem( services.first()->name() + " - " + i18n( "Cylinder" ));
         monitorAddItem( services.first()->name() + " - " + i18n( "Sphere" ));
         }
+    services = trader->query( "KWin/Effect", "[X-KDE-PluginInfo-Name] == 'kwin4_effect_flipswitch'" );
+    if( services.isEmpty() )
+        {
+        // adding empty strings in case the effect is not found
+        // TODO: after string freeze add a info that the effect is missing
+        monitorAddItem( QString() );
+        monitorAddItem( QString() );
+        }
+    else
+        {
+        monitorAddItem( services.first()->name() + " - " + i18n( "All Desktops" ));
+        monitorAddItem( services.first()->name() + " - " + i18n( "Current Desktop" ));
+        }
 
     monitorShowEvent();
     }
@@ -290,6 +318,25 @@ void KWinScreenEdgesConfig::monitorLoad()
         {
         monitorChangeEdge( ElectricBorder( i ), int( Sphere ) );
         }
+
+    // Flip Switch
+    KConfigGroup flipSwitchConfig( m_config, "Effect-FlipSwitch" );
+    list.clear();
+    // FlipSwitch BorderActivateAll
+    list.append( int( ElectricNone ) );
+    list = flipSwitchConfig.readEntry( "BorderActivateAll", list );
+    foreach( int i, list )
+        {
+        monitorChangeEdge( ElectricBorder( i ), int( FlipSwitchAll ) );
+        }
+    // FlipSwitch BorderActivate
+    list.clear();
+    list.append( int( ElectricNone ) );
+    list = flipSwitchConfig.readEntry( "BorderActivate", list );
+    foreach( int i, list )
+        {
+        monitorChangeEdge( ElectricBorder( i ), int( FlipSwitchCurrent ) );
+        }
     }
 
 void KWinScreenEdgesConfig::monitorSaveAction( int edge, const QString& configName )
@@ -298,6 +345,8 @@ void KWinScreenEdgesConfig::monitorSaveAction( int edge, const QString& configNa
     int item = m_ui->monitor->selectedEdgeItem( edge );
     if( item == 1 ) // Plasma dashboard
         config.writeEntry( configName, "Dashboard" );
+    else if( item == 2 )
+        config.writeEntry( configName, "ShowDesktop" );
     else // Anything else
         config.writeEntry( configName, "None" );
     }
@@ -336,6 +385,13 @@ void KWinScreenEdgesConfig::monitorSave()
         monitorCheckEffectHasEdge( int( Cylinder )));
     cubeConfig.writeEntry( "BorderActivateSphere",
         monitorCheckEffectHasEdge( int( Sphere )));
+
+    // Flip Switch
+    KConfigGroup flipSwitchConfig( m_config, "Effect-FlipSwitch" );
+    flipSwitchConfig.writeEntry( "BorderActivateAll",
+        monitorCheckEffectHasEdge( int( FlipSwitchAll )));
+    flipSwitchConfig.writeEntry( "BorderActivate",
+        monitorCheckEffectHasEdge( int( FlipSwitchCurrent )));
     }
 
 void KWinScreenEdgesConfig::monitorDefaults()
@@ -352,7 +408,7 @@ void KWinScreenEdgesConfig::monitorShowEvent()
     {
     // Check if they are enabled
     KConfigGroup config( m_config, "Compositing" );
-    if( config.readEntry( "Enabled", m_defaultPrefs.enableCompositing() ))
+    if( config.readEntry( "Enabled", m_defaultPrefs.recommendCompositing() ))
         { // Compositing enabled
         config = KConfigGroup( m_config, "Plugins" );
 
@@ -370,6 +426,11 @@ void KWinScreenEdgesConfig::monitorShowEvent()
         monitorItemSetEnabled( int( Cube ), enabled );
         monitorItemSetEnabled( int( Cylinder ), enabled );
         monitorItemSetEnabled( int( Sphere ), enabled );
+
+        // Flip Switch
+        enabled = effectEnabled( "flipswitch", config );
+        monitorItemSetEnabled( int( FlipSwitchAll ), enabled );
+        monitorItemSetEnabled( int( FlipSwitchCurrent ), enabled );
         }
     else // Compositing disabled
         {
@@ -379,6 +440,8 @@ void KWinScreenEdgesConfig::monitorShowEvent()
         monitorItemSetEnabled( int( Cube ), false );
         monitorItemSetEnabled( int( Cylinder ), false );
         monitorItemSetEnabled( int( Sphere ), false );
+        monitorItemSetEnabled( int( FlipSwitchAll ), false );
+        monitorItemSetEnabled( int( FlipSwitchCurrent ), false );
         }
     }
 
