@@ -456,7 +456,9 @@ void Scene::Window::resetPaintingEnabled()
         {
         if( c->isMinimized() )
             disable_painting |= PAINT_DISABLED_BY_MINIMIZE;
-        if( c->isHiddenInternal())
+        if( c->clientGroup() && c != c->clientGroup()->visible() )
+            disable_painting |= PAINT_DISABLED_BY_CLIENT_GROUP;
+        else if( c->isHiddenInternal())
             disable_painting |= PAINT_DISABLED;
         }
     }
@@ -482,10 +484,22 @@ WindowQuadList Scene::Window::buildQuads( bool force ) const
         {
         Client *client = dynamic_cast<Client*>( toplevel );
         QRegion contents = clientShape();
+        QRegion center = toplevel->transparentRect();
         QRegion decoration = (client && Workspace::self()->decorationHasAlpha() ?
-                              QRegion(client->decorationRect()) : shape()) - contents;
+                              QRegion(client->decorationRect()) : shape()) - center;
         ret = makeQuads( WindowQuadContents, contents );
-        ret += makeQuads( WindowQuadDecoration, decoration );
+        if( !client || !(center.isEmpty() || client->isShade()) )
+            ret += makeQuads( WindowQuadDecoration, decoration );
+        else
+            {
+            // this is a shaded client, we have to create four decoartion quads
+            QRect left, top, right, bottom;
+            client->layoutDecorationRects( left, top, right, bottom, Client::WindowRelative );
+            ret += makeQuads( WindowQuadDecoration, top );
+            ret += makeQuads( WindowQuadDecoration, bottom );
+            ret += makeQuads( WindowQuadDecoration, left );
+            ret += makeQuads( WindowQuadDecoration, right );
+            }
         }
     effects->buildQuads( static_cast<Client*>( toplevel )->effectWindow(), ret );
     cached_quad_list = new WindowQuadList( ret );
