@@ -54,8 +54,13 @@ class QPushButton;
 namespace KWin
 {
 
-class Client;
+namespace TabBox
+{
 class TabBox;
+}
+
+class Client;
+class ClientGroup;
 class DesktopChangeOSD;
 class RootInfo;
 class PluginMgr;
@@ -140,7 +145,7 @@ class Workspace : public QObject, public KDecorationDefines
         void place( Client* c, QRect& area );
         void placeSmart( Client* c, const QRect& area );
 
-        QPoint adjustClientPosition( Client* c, QPoint pos, bool unrestricted );
+        QPoint adjustClientPosition( Client* c, QPoint pos, bool unrestricted, double snapAdjust = 1.0 );
         QRect adjustClientSize( Client* c, QRect moveResizeGeom, int mode );
         void raiseClient( Client* c, bool nogroup = false );
         void lowerClient( Client* c, bool nogroup = false );
@@ -157,6 +162,7 @@ class Workspace : public QObject, public KDecorationDefines
         void clientAttentionChanged( Client* c, bool set );
 
         void checkElectricBorder(const QPoint& pos, Time time);
+        void restoreElectricBorderSize( ElectricBorder border );
         void reserveElectricBorder( ElectricBorder border );
         void unreserveElectricBorder( ElectricBorder border );
         void reserveElectricBorderActions( bool reserve );
@@ -167,7 +173,7 @@ class Workspace : public QObject, public KDecorationDefines
 
     public:
         /**
-         * @returns Total number of desktops currently in existance.
+         * @returns Total number of desktops currently in existence.
          */
         int numberOfDesktops() const;
         /**
@@ -309,6 +315,16 @@ class Workspace : public QObject, public KDecorationDefines
         void unrefTabBox();
         void closeTabBox();
 
+        // Tabbing
+        void addClientGroup( ClientGroup* group );
+        void removeClientGroup( ClientGroup* group );
+        /// Returns the index of c in clientGroupList.
+        int indexOfClientGroup( ClientGroup* group );
+        /// Change the client c_id to the group with index g_id
+        void moveItemToClientGroup( ClientGroup* oldGroup, int oldIndex, ClientGroup* group, int index = -1 );
+        Client* findSimilarClient( Client* c );
+        QList<ClientGroup*> clientGroups; // List of existing clients groups with no special order
+
         /**
          * Returns the list of clients sorted in stacking order, with topmost client
          * at the last position
@@ -366,6 +382,8 @@ class Workspace : public QObject, public KDecorationDefines
 
         bool hasDecorationShadows() const;
         bool decorationHasAlpha() const;
+        bool decorationSupportsClientGrouping() const; // Returns true if the decoration supports tabs.
+        bool decorationSupportsFrameOverlap() const;
 
         // D-Bus interface
         void cascadeDesktop();
@@ -467,6 +485,8 @@ class Workspace : public QObject, public KDecorationDefines
         void stopMousePolling();
 
         void raiseElectricBorderWindows();
+        void showElectricBorderWindowOutline();
+        void hideElectricBorderWindowOutline();
 
     public slots:
         void addRepaintFull();
@@ -544,6 +564,7 @@ class Workspace : public QObject, public KDecorationDefines
         void slotWindowToScreen7() { return slotWindowToScreen( 7 ); }
         void slotSwitchToNextScreen();
         void slotWindowToNextScreen();
+        void slotToggleShowDesktop();
 
         void slotWindowMaximize();
         void slotWindowMaximizeVertical();
@@ -562,6 +583,8 @@ class Workspace : public QObject, public KDecorationDefines
         void slotWindowGrowVertical();
         void slotWindowShrinkHorizontal();
         void slotWindowShrinkVertical();
+        void slotWindowQuickTileLeft();
+        void slotWindowQuickTileRight();
 
         void slotWalkThroughDesktops();
         void slotWalkBackThroughDesktops();
@@ -569,6 +592,8 @@ class Workspace : public QObject, public KDecorationDefines
         void slotWalkBackThroughDesktopList();
         void slotWalkThroughWindows();
         void slotWalkBackThroughWindows();
+        void slotWalkThroughWindowsAlternative();
+        void slotWalkBackThroughWindowsAlternative();
 
         void slotWalkThroughDesktopsKeyChanged( const QKeySequence& seq );
         void slotWalkBackThroughDesktopsKeyChanged( const QKeySequence& seq );
@@ -576,6 +601,15 @@ class Workspace : public QObject, public KDecorationDefines
         void slotWalkBackThroughDesktopListKeyChanged( const QKeySequence& seq );
         void slotWalkThroughWindowsKeyChanged( const QKeySequence& seq );
         void slotWalkBackThroughWindowsKeyChanged( const QKeySequence& seq );
+        void slotMoveToTabLeftKeyChanged( const QKeySequence& seq );
+        void slotMoveToTabRightKeyChanged( const QKeySequence& seq );
+        void slotWalkThroughWindowsAlternativeKeyChanged( const QKeySequence& seq );
+        void slotWalkBackThroughWindowsAlternativeKeyChanged( const QKeySequence& seq );
+
+        void slotSwitchWindowUp();
+        void slotSwitchWindowDown();
+        void slotSwitchWindowRight();
+        void slotSwitchWindowLeft();
 
         void slotWindowOperations();
         void slotWindowClose();
@@ -617,7 +651,15 @@ class Workspace : public QObject, public KDecorationDefines
         void suspendCompositing();
         void suspendCompositing( bool suspend );
 
+        void slotSwitchToTabLeft(); // Slot to move left the active Client.
+        void slotSwitchToTabRight(); // Slot to move right the active Client.
+        void slotRemoveFromGroup(); // Slot to remove the active client from its group.
+
     private slots:
+        void groupTabPopupAboutToShow(); // Popup to add to another group
+        void switchToTabPopupAboutToShow(); // Popup to move in the group
+        void slotAddToTabGroup( QAction* ); // Add client to a group
+        void slotSwitchToTab( QAction* ); // Change the tab
         void desktopPopupAboutToShow();
         void clientPopupAboutToShow();
         void slotSendToDesktop( QAction* );
@@ -658,14 +700,23 @@ class Workspace : public QObject, public KDecorationDefines
         void setupWindowShortcut( Client* c );
         void checkCursorPos();
 
-        bool startKDEWalkThroughWindows();
+        enum Direction
+            {
+            DirectionNorth,
+            DirectionEast,
+            DirectionSouth,
+            DirectionWest
+            };
+        void switchWindow( Direction direction );
+        bool startKDEWalkThroughWindows( TabBoxMode mode ); // TabBoxWindowsMode | TabBoxWindowsAlternativeMode
         bool startWalkThroughDesktops( TabBoxMode mode ); // TabBoxDesktopMode | TabBoxDesktopListMode
         bool startWalkThroughDesktops();
         bool startWalkThroughDesktopList();
+        void navigatingThroughWindows( bool forward, const KShortcut& shortcut, TabBoxMode mode ); // TabBoxWindowsMode | TabBoxWindowsAlternativeMode
         void KDEWalkThroughWindows( bool forward );
         void CDEWalkThroughWindows( bool forward );
         void walkThroughDesktops( bool forward );
-        void KDEOneStepThroughWindows( bool forward );
+        void KDEOneStepThroughWindows( bool forward, TabBoxMode mode ); // TabBoxWindowsMode | TabBoxWindowsAlternativeMode
         void oneStepThroughDesktops( bool forward, TabBoxMode mode ); // TabBoxDesktopMode | TabBoxDesktopListMode
         void oneStepThroughDesktops( bool forward );
         void oneStepThroughDesktopList( bool forward );
@@ -792,18 +843,22 @@ class Workspace : public QObject, public KDecorationDefines
         KShortcut cutWalkThroughDesktops, cutWalkThroughDesktopsReverse;
         KShortcut cutWalkThroughDesktopList, cutWalkThroughDesktopListReverse;
         KShortcut cutWalkThroughWindows, cutWalkThroughWindowsReverse;
+        KShortcut cutWalkThroughGroupWindows, cutWalkThroughGroupWindowsReverse;
+        KShortcut cutWalkThroughWindowsAlternative, cutWalkThroughWindowsAlternativeReverse;
         bool mouse_emulation;
         unsigned int mouse_emulation_state;
         WId mouse_emulation_window;
         int block_focus;
 
-        TabBox* tab_box;
+        TabBox::TabBox* tab_box;
         DesktopChangeOSD* desktop_change_osd;
 
         QMenu* popup;
         QMenu* advanced_popup;
         QMenu* trans_popup;
         QMenu* desk_popup;
+        QMenu* add_tabs_popup; // Menu to add the group to other group
+        QMenu* switch_to_tab_popup; // Menu to change tab
 
         void modalActionsSwitch( bool enabled );
 
@@ -819,12 +874,17 @@ class Workspace : public QObject, public KDecorationDefines
         QAction* mNoBorderOpAction;
         QAction* mMinimizeOpAction;
         QAction* mCloseOpAction;
+        QAction* mRemoveTabGroup; // Remove client from group
+        QAction* mCloseGroup; // Close all clients in the group
         ShortcutDialog* client_keys_dialog;
         Client* client_keys_client;
         KActionCollection* disable_shortcuts_keys;
         bool global_shortcuts_disabled;
         bool global_shortcuts_disabled_for_client;
 
+        void initAddToTabGroup(); // Load options for menu add_tabs_popup
+        void initSwitchToTab(); // Load options for menu switch_to_tab_popup
+
         PluginMgr* mgr;
 
         RootInfo* rootInfo;
@@ -903,6 +963,11 @@ class Workspace : public QObject, public KDecorationDefines
         QList< int > composite_paint_times;
         QTimer compositeResetTimer; // for compressing composite resets
 
+        Window outline_left;
+        Window outline_right;
+        Window outline_top;
+        Window outline_bottom;
+
     private:
         friend bool performTransiencyCheck();
     };
@@ -1188,6 +1253,26 @@ inline bool Workspace::decorationHasAlpha() const
     return mgr->factory()->supports( AbilityUsesAlphaChannel );
     }
 
+inline bool Workspace::decorationSupportsClientGrouping() const
+    {
+    return mgr->factory()->supports( AbilityClientGrouping );
+    }
+
+inline bool Workspace::decorationSupportsFrameOverlap() const
+    {
+    return mgr->factory()->supports( AbilityExtendIntoClientArea );
+    }
+
+inline void Workspace::addClientGroup( ClientGroup* group )
+    {
+    clientGroups.append( group );
+    }
+
+inline void Workspace::removeClientGroup( ClientGroup* group )
+    {
+    clientGroups.removeAll( group );
+    }
+
 } // namespace
 
 #endif
