@@ -60,10 +60,6 @@
 #include "ui_ProcessWidgetUI.h"
 #include "scripting.h"
 
-#ifdef WITH_MONITOR_PROCESS_IO
-#include "DisplayProcessDlg.h"
-#endif
-
 #include <sys/types.h>
 #include <unistd.h>
 
@@ -159,17 +155,13 @@ struct KSysGuardProcessListPrivate {
     {
         mScripting = NULL;
         mNumItemsSelected = -1;
+        mResortCountDown = 2; //The items added initially will be already sorted, but without CPU info.  On the second refresh we will have CPU usage, so /then/ we can resort
         renice = new KAction(i18np("Renice Process...", "Renice Processes...", 1), q);
         renice->setShortcut(Qt::Key_F8);
         selectParent = new KAction(i18n("Jump to Parent Process"), q);
 
         selectTracer = new KAction(i18n("Jump to Process Debugging This One"), q);
         window = new KAction(i18n("Show Application Window"), q);
-#ifdef WITH_MONITOR_PROCESS_IO
-        monitorio = new KAction(i18n("Monitor Input && Output"), q);
-#else
-        monitorio = 0;
-#endif
         resume = new KAction(i18n("Resume Stopped Process"), q);
         kill = new KAction(i18np("Kill Process...", "Kill Processes...", 1), q);
         kill->setIcon(KIcon("process-stop"));
@@ -227,13 +219,15 @@ struct KSysGuardProcessListPrivate {
     /** Class to deal with the scripting. NULL if scripting is disabled */
     Scripting *mScripting;
 
+    /** A counter to mark when to resort, so that we do not resort on every update */
+    int mResortCountDown;
+
     KAction *renice;
     KAction *kill;
     KAction *selectParent;
     KAction *selectTracer;
     KAction *jumpToSearchFilter;
     KAction *window;
-    KAction *monitorio;
     KAction *resume;
     KAction *sigStop;
     KAction *sigCont;
@@ -320,8 +314,6 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
     QSignalMapper *signalMapper = new QSignalMapper(this);
     QList<QAction *> actions;
     actions << d->renice << d->kill << d->selectParent << d->selectTracer << d->window << d->jumpToSearchFilter;
-    if (d->monitorio)
-        actions << d->monitorio;
     actions << d->resume << d->sigStop << d->sigCont << d->sigHup << d->sigInt << d->sigTerm << d->sigKill << d->sigUsr1 << d->sigUsr2;
 
     foreach(QAction *action, actions) {
@@ -333,16 +325,7 @@ KSysGuardProcessList::KSysGuardProcessList(QWidget* parent, const QString &hostN
 
     retranslateUi();
 
-    d->mFilterModel.setDynamicSortFilter(true);
-
     d->mUi->btnKillProcess->setIcon(KIcon("process-stop"));
-
-    //If the view resorts continually, then it can be hard to keep track of processes.  By doing it only every few seconds it reduces the 'jumping around'
-    //	QTimer *mTimer = new QTimer(this);
-    //	mTimer->start(4000);
-    // QT BUG?  We have to disable the sorting for now because there seems to be a bug in Qt introduced in Qt 4.4beta which makes the view scroll back to the top
-    //    updateList();
-    d->mModel.update(d->mUpdateIntervalMSecs);
 }
 
 KSysGuardProcessList::~KSysGuardProcessList()
@@ -477,9 +460,6 @@ void KSysGuardProcessList::showProcessContextMenu(const QPoint &point) {
     if (numProcesses == 1 && !d->mModel.data(realIndex, ProcessModel::WindowIdRole).isNull()) {
         d->mProcessContextMenu->addAction(d->window);
     }
-    if (d->monitorio && numProcesses == 1 && d->mModel.isLocalhost() && (process->uid==0 || process->uid == getuid()) && process->pid != getpid() && process->pid != getppid()) { //Don't attach to ourselves - crashes
-        d->mProcessContextMenu->addAction(d->monitorio);
-    }
 
     if(numProcesses == 1 && process->status == KSysGuard::Process::Stopped) {
         //If the process is stopped, offer to resume it
@@ -540,17 +520,6 @@ void KSysGuardProcessList::actionTriggered(QObject *object) {
         }
     } else if(result == d->jumpToSearchFilter) {
         d->mUi->txtFilter->setFocus();
-#ifdef WITH_MONITOR_PROCESS_IO
-    } else if(result == d->monitorio) {
-        QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
-        if(selectedIndexes.isEmpty()) return;  //No processes selected
-        QModelIndex realIndex = d->mFilterModel.mapToSource(selectedIndexes.at(0));
-        KSysGuard::Process *process = reinterpret_cast<KSysGuard::Process *> (realIndex.internalPointer());
-        if(process) {
-            DisplayProcessDlg *dialog = new DisplayProcessDlg( this,process );
-            dialog->show();
-        }
-#endif
     } else {
         QList< long long > pidlist;
         QModelIndexList selectedIndexes = d->mUi->treeView->selectionModel()->selectedRows();
@@ -901,10 +870,7 @@ void KSysGuardProcessList::hideEvent ( QHideEvent * event )  //virtual protected
 void KSysGuardProcessList::showEvent ( QShowEvent * event )  //virtual protected from QWidget
 {
     //Start updating the process list again if we are shown again
-    if(d->mUpdateTimer && !d->mUpdateTimer->isActive()) {
-        d->mUpdateTimer->start(d->mUpdateIntervalMSecs);
-    }
-
+    updateList();
     QWidget::showEvent(event);
 }
 
@@ -917,7 +883,6 @@ void KSysGuardProcessList::changeEvent( QEvent * event )
     }
     QWidget::changeEvent(event);
 }
-
 void KSysGuardProcessList::retranslateUi()
 {
     d->mUi->cmbFilter->setItemIcon(ProcessFilter::AllProcesses, KIcon("view-process-all"));
@@ -945,6 +910,13 @@ void KSysGuardProcessList::updateList()
                 qApp->notify(w, &event);
             }
         }
+        if(--d->mResortCountDown <= 0) {
+            d->mResortCountDown = 2;  //resort every second time
+            //resort now
+            QHeaderView *header= d->mUi->treeView->header();
+            d->mUi->treeView->sortByColumn(header->sortIndicatorSection(), header->sortIndicatorOrder());
+
+        }
     }
 }
 
@@ -964,6 +936,8 @@ void KSysGuardProcessList::setUpdateIntervalMSecs(int intervalMSecs)
         d->mUpdateTimer = NULL;
         return;
     }
+    if(!isVisible())
+        return;
 
     if(!d->mUpdateTimer) {
         //intervalMSecs is a valid time, so set up a timer
