@@ -23,8 +23,9 @@
 #include "soliddefs_p.h"
 #include "managerbase_p.h"
 #include "ifaces/powermanager.h"
-
+#include <kdebug.h>
 #include <kglobal.h>
+#include <QX11Info>
 
 K_GLOBAL_STATIC(Solid::Control::PowerManagerPrivate, globalPowerManager)
 
@@ -147,14 +148,20 @@ Solid::Control::PowerManager::BrightnessControlsList Solid::Control::PowerManage
                       Solid::Control::PowerManager::BrightnessControlsList(), brightnessControlsAvailable());
 }
 
+extern float xrandr_brightlight(Display *dpy, long new_value = -1);
+
 bool Solid::Control::PowerManager::setBrightness(float brightness, const QString &device)
 {
     if(device.isEmpty())
     {
         Solid::Control::PowerManager::BrightnessControlsList controls = brightnessControlsAvailable();
-        if(controls.size() == 0)
+        if(controls.keys(Solid::Control::PowerManager::Screen).isEmpty())
         {
+#ifdef Q_WS_WIN
             return false;
+#else
+            return ( xrandr_brightlight( QX11Info::display(), brightness ) >= 0 );
+#endif
         }
         else
         {
@@ -178,14 +185,18 @@ float Solid::Control::PowerManager::brightness(const QString &device)
     if(device.isEmpty())
     {
         Solid::Control::PowerManager::BrightnessControlsList controls = brightnessControlsAvailable();
-        if(controls.size() == 0)
+        if(controls.keys(Solid::Control::PowerManager::Screen).isEmpty())
         {
+#ifdef Q_WS_WIN
             return false;
+#else
+            return xrandr_brightlight( QX11Info::display() );
+#endif
         }
         else
         {
             return_SOLID_CALL(Ifaces::PowerManager *, globalPowerManager->managerBackend(),
-                      false, brightness(controls.keys().at(0)));
+                      false, brightness(controls.keys(Solid::Control::PowerManager::Screen).at(0)));
         }
     }
     else
